package sun.plugin.security;

import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;
import sun.plugin.resources.ResourceHandler;
import sun.plugin.usability.Trace;

public final class X509PluginTrustManager implements X509TrustManager {
   private X509TrustManager trustManager = null;
   private static CertificateStore rootStore = null;
   private static CertificateStore httpsRootStore = null;
   private static CertificateStore permanentStore = null;
   private static CertificateStore sessionStore = null;
   private static CertificateStore deniedStore = null;

   public static void reset() {
      rootStore = new RootCACertificateStore();
      httpsRootStore = new HttpsRootCACertStore();
      permanentStore = new PluginHttpsCertStore();
      sessionStore = new SessionCertificateStore();
      deniedStore = new DeniedCertificateStore();
   }

   public X509PluginTrustManager() throws KeyStoreException, NoSuchAlgorithmException, NoSuchProviderException, CertificateException {
      TrustManagerFactory var1 = TrustManagerFactory.getInstance("SunX509", "SunJSSE");
      var1.init((KeyStore)null);
      TrustManager[] var2 = var1.getTrustManagers();
      this.trustManager = (X509TrustManager)var2[0];
   }

   public void checkClientTrusted(X509Certificate[] var1, String var2) throws CertificateException {
      boolean var3 = false;
      boolean var4 = false;
      if (!$assertionsDisabled && this.trustManager == null) {
         throw new AssertionError();
      } else {
         int var5 = 1;

         try {
            rootStore.load();
            httpsRootStore.load();
            permanentStore.load();
            sessionStore.load();
            deniedStore.load();
            if (deniedStore.contains(var1[0])) {
               throw new CertificateException("Certificate has been denied");
            }

            try {
               this.trustManager.checkClientTrusted(var1, var2);
               return;
            } catch (CertificateException var11) {
               if (sessionStore.contains(var1[0])) {
                  return;
               }

               if (permanentStore.contains(var1[0])) {
                  return;
               }

               for (int var6 = 0; var6 < var1.length; var6++) {
                  if (!rootStore.verify(var1[var6]) && !httpsRootStore.verify(var1[var6])) {
                     var3 = true;
                  }

                  try {
                     var1[var6].checkValidity();
                  } catch (CertificateExpiredException var9) {
                     var4 = true;
                  } catch (CertificateNotYetValidException var10) {
                     var4 = true;
                  }
               }

               if (!Trace.isAutomationEnabled()) {
                  TrustDeciderDialog var7 = new TrustDeciderDialog(var1, var3, var4);
                  var7.setHttpsDialog(true);
                  var5 = var7.DoModal();
               } else {
                  Trace.securityPrintln(getMessage("x509trustmgr.automation.ignoreclientcert"), 2);
                  var5 = 0;
               }

               if (var5 == 0) {
                  sessionStore.add(var1[0]);
                  sessionStore.save();
               } else if (var5 == 2) {
                  permanentStore.add(var1[0]);
                  permanentStore.save();
               } else {
                  deniedStore.add(var1[0]);
                  deniedStore.save();
               }
            }
         } catch (CertificateException var12) {
            throw var12;
         } catch (Throwable var13) {
            var13.printStackTrace();
         }

         if (var5 != 0 && var5 != 2) {
            throw new CertificateException("Plugin couldn't trust Client");
         }
      }
   }

   public void checkServerTrusted(X509Certificate[] var1, String var2) throws CertificateException {
      boolean var3 = false;
      boolean var4 = false;
      if (!$assertionsDisabled && this.trustManager == null) {
         throw new AssertionError();
      } else {
         int var5 = 1;

         try {
            rootStore.load();
            httpsRootStore.load();
            permanentStore.load();
            sessionStore.load();
            deniedStore.load();
            if (deniedStore.contains(var1[0])) {
               throw new CertificateException("Certificate has been denied");
            }

            try {
               this.trustManager.checkServerTrusted(var1, var2);
               return;
            } catch (CertificateException var11) {
               if (sessionStore.contains(var1[0])) {
                  return;
               }

               if (permanentStore.contains(var1[0])) {
                  return;
               }

               for (int var6 = 0; var6 < var1.length; var6++) {
                  if (!rootStore.verify(var1[var6]) && !httpsRootStore.verify(var1[var6])) {
                     var3 = true;
                  }

                  try {
                     var1[var6].checkValidity();
                  } catch (CertificateExpiredException var9) {
                     var4 = true;
                  } catch (CertificateNotYetValidException var10) {
                     var4 = true;
                  }
               }

               if (!Trace.isAutomationEnabled()) {
                  TrustDeciderDialog var7 = new TrustDeciderDialog(var1, var3, var4);
                  var7.setHttpsDialog(true);
                  var5 = var7.DoModal();
               } else {
                  Trace.securityPrintln(getMessage("x509trustmgr.automation.ignoreservercert"), 2);
                  var5 = 0;
               }

               if (var5 == 0) {
                  sessionStore.add(var1[0]);
                  sessionStore.save();
               } else if (var5 == 2) {
                  permanentStore.add(var1[0]);
                  permanentStore.save();
               } else {
                  deniedStore.add(var1[0]);
                  deniedStore.save();
               }
            }
         } catch (CertificateException var12) {
            throw var12;
         } catch (Throwable var13) {
            var13.printStackTrace();
         }

         if (var5 != 0 && var5 != 2) {
            throw new CertificateException("Plugin couldn't trust Server");
         }
      }
   }

   public X509Certificate[] getAcceptedIssuers() {
      return this.trustManager.getAcceptedIssuers();
   }

   private static String getMessage(String var0) {
      return ResourceHandler.getMessage(var0);
   }

   static {
      reset();
   }
}
