package com.raritan.smartcard.impl;

import com.raritan.smartcard.CardAccessErrorsListener;
import com.raritan.smartcard.SmartCardReaderStatus;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.smartcardio.Card;
import javax.smartcardio.CardException;
import javax.smartcardio.CardNotPresentException;
import javax.smartcardio.CardTerminal;
import javax.smartcardio.CommandAPDU;
import javax.smartcardio.ResponseAPDU;
import nn.pp.core.WorkstationUnlockDetector;

public class CardAccessHandler implements SmartCardReaderListener16, CardTransmitHandler {
   private static final int FALSE_ALARM_PREVENTION_TIMEOUT = 1000;
   private static final Logger LOGGER = Logger.getLogger(CardAccessHandler.class.getName());
   private CardAccessHandler.CardAccessData cardAccessData;
   private final CardLockProvider lockProvider;
   private final TransportProtoProvider transProtoProvider;
   private final ServerCardEventsListener serverCardEventsList;
   private final CardAccessErrorsListener cardAccessErrorsList;
   private final CardEventsStateMachine cardStateMachine;
   private final WorkstationUnlockDetector workstationUnlockDetector;
   private final ExecutorService service;
   private final ThreadLocal<Card> thrLocalCard = new ThreadLocal<>();
   private final ThreadLocal<Lock> thrLocalLock = new ThreadLocal<>();

   private void disconnectCard(Card var1, boolean var2) {
      try {
         var1.disconnect(var2);
      } catch (CardException var4) {
         LOGGER.log(Level.INFO, "Exception on disconnect ", (Throwable)var4);
      }
   }

   private void endTransaction(Card var1, Lock var2) throws CardException, CardResetException {
      try {
         var1.endExclusive();
      } catch (CardException var7) {
         LOGGER.log(Level.INFO, "Exception on endExclusive ", (Throwable)var7);
         if (isCardReset(var7.getCause())) {
            this.disconnectCard(var1, false);
            this.lockProvider.invalidateCard(var1);
            throw new CardResetException(var7);
         }

         throw var7;
      } finally {
         this.disconnectCard(var1, true);
         this.lockProvider.invalidateCard(var1);
         this.cleanup(var2);
      }
   }

   private void cleanup(Lock var1) {
      this.thrLocalLock.remove();
      this.thrLocalCard.remove();
      var1.unlock();
   }

   public CardAccessHandler(
      CardLockProvider var1,
      TransportProtoProvider var2,
      ServerCardEventsListener var3,
      CardAccessErrorsListener var4,
      WorkstationUnlockDetector var5,
      String var6
   ) {
      this.lockProvider = var1;
      this.transProtoProvider = var2;
      this.serverCardEventsList = var3;
      this.cardAccessErrorsList = var4;
      this.cardStateMachine = new CardEventsStateMachine();
      this.workstationUnlockDetector = var5;
      ThreadPoolExecutor var7 = new ThreadPoolExecutor(
         1, 1, 3000L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<>(), new CardAccessHandler.CardAccessHandlerThrFactory(CardAccessHandler.class, var6)
      );
      var7.allowCoreThreadTimeOut(true);
      this.service = var7;
   }

   private List performCardConnectAction(CardTerminal var1) throws CardResetException, InterruptedException, CardNotPresentException {
      int var2 = 0;
      Card var3 = null;

      while (var2 < TransportProtoProvider.PROTO.length) {
         try {
            var3 = this.getCard(var1, TransportProtoProvider.PROTO[var2]);
            break;
         } catch (CardNotPresentException var5) {
            throw var5;
         } catch (CardException var6) {
            var2++;
            LOGGER.log(Level.INFO, "Exception on connecting to card ", (Throwable)var6);
         }
      }

      if (var3 != null) {
         ArrayList var4 = new ArrayList();
         var4.add(var3);
         var4.add(var2);
         return var4;
      } else {
         return null;
      }
   }

   private CardAccessHandler.CardAccessData performCardDataAction(CardTerminal var1, int var2, Card var3) throws CardResetException, InterruptedException, IllegalStateException {
      return this.createCardAccessData(var3, var2, var1);
   }

   @Override
   public void cardInserted(CardTerminal var1) {
      try {
         this.waitTillWorkstationUnlocked();
      } catch (InterruptedException var9) {
         Thread.currentThread().interrupt();
         return;
      }

      while (true) {
         List var2;
         while (true) {
            try {
               var2 = this.performCardConnectAction(var1);
               break;
            } catch (CardResetException var10) {
               LOGGER.log(Level.INFO, "Card Reset during CardConnectAction ", (Throwable)var10);
            } catch (InterruptedException var11) {
               Thread.currentThread().interrupt();
               return;
            } catch (CardNotPresentException var12) {
               return;
            }
         }

         int var5 = 0;
         Card var6 = null;
         if (var2 != null) {
            var6 = (Card)var2.get(0);
            var5 = (Integer)var2.get(1);
         }

         if (var6 == null) {
            this.cardAccessErrorsList.noProtocolSupported(var1.getName());
            return;
         }

         LOGGER.log(Level.INFO, "Card supports Protocol " + TransportProtoProvider.PROTO[var5]);

         String var3;
         byte[] var4;
         try {
            CardAccessHandler.CardAccessData var7;
            try {
               var7 = this.performCardDataAction(var1, var5, var6);
            } catch (CardResetException var13) {
               LOGGER.log(Level.INFO, "Card Reset during CardDataAction ", (Throwable)var13);
               continue;
            }

            this.setCardAccessData(var7);
            var3 = TransportProtoProvider.PROTO[var5];
            var4 = this.getCardAccessData().getAtrBytes();
         } catch (InterruptedException var14) {
            Thread.currentThread().interrupt();
            return;
         } catch (IllegalStateException var15) {
            LOGGER.log(Level.INFO, "Exception on accessing card ", (Throwable)var15);
            return;
         }

         if (this.cardStateMachine.setCardStatus(SmartCardReaderStatus.PRESENT)) {
            this.serverCardEventsList.cardInserted(var3, var4);
         } else {
            assert false : "Incorrect card state";
         }

         return;
      }
   }

   @Override
   public void cardReaderRemoved(String var1) {
      this.reset();
      this.service.shutdown();
      this.serverCardEventsList.cardReaderRemoved();
   }

   @Override
   public void cardRemoved(CardTerminal var1) {
      CardAccessHandler.CardAccessData var2 = this.getCardAccessData();
      if (var2 != null) {
         try {
            var2.card.disconnect(false);
         } catch (CardException var4) {
            LOGGER.log(Level.INFO, "exception on disconnect", (Throwable)var4);
         }
      }

      this.reset();
      if (this.cardStateMachine.setCardStatus(SmartCardReaderStatus.ABSENT)) {
         this.serverCardEventsList.cardRemoved();
      }
   }

   @Override
   public void transmit(byte[] var1) {
      CardAccessHandler.CardAccessData var2 = this.getCardAccessData();
      if (var2 != null) {
         CardTerminal var3 = var2.getTerminal();

         while (true) {
            try {
               this.transmit0(var1);
               return;
            } catch (CardResetException var16) {
               LOGGER.log(Level.INFO, "Card Reset during transmit0 ", (Throwable)var16);

               while (true) {
                  List var5;
                  try {
                     var5 = this.performCardConnectAction(var3);
                  } catch (CardResetException var13) {
                     LOGGER.log(Level.INFO, "Card Reset during CardConnectAction ", (Throwable)var13);
                     continue;
                  } catch (InterruptedException var14) {
                     Thread.currentThread().interrupt();
                     return;
                  } catch (CardNotPresentException var15) {
                     this.handleTransmitError(var3);
                     return;
                  }

                  int var6 = 0;
                  Object var7 = null;
                  if (var5 != null) {
                     var7 = (Card)var5.get(0);
                     var6 = (Integer)var5.get(1);

                     try {
                        CardAccessHandler.CardAccessData var8;
                        try {
                           var8 = this.performCardDataAction(var3, var6, (Card)var7);
                        } catch (CardResetException var10) {
                           LOGGER.log(Level.INFO, "Card Reset during CardDataAction ", (Throwable)var10);
                           continue;
                        }

                        this.setCardAccessData(var8);
                        break;
                     } catch (InterruptedException var11) {
                        Thread.currentThread().interrupt();
                        return;
                     } catch (IllegalStateException var12) {
                        this.handleTransmitError(var3);
                        return;
                     }
                  }

                  this.handleTransmitError(var3);
                  this.cardAccessErrorsList.noProtocolSupported(var3.getName());
                  return;
               }
            }
         }
      }
   }

   public void transmit0(byte[] var1) throws CardResetException {
      if (this.cardStateMachine.getCardState().equals(SmartCardReaderStatus.PRESENT)) {
         CardAccessHandler.CardAccessData var2 = this.getCardAccessData();
         Card var3 = var2.getCard();
         CardTerminal var4 = var2.getTerminal();
         TransportProtoHandler var5 = var2.getTransportProtoHandler();

         byte[] var6;
         try {
            var6 = this.transmitOnWorkerThread(var3, var1, var5);
         } catch (InterruptedException var8) {
            Thread.currentThread().interrupt();
            return;
         } catch (CardException var9) {
            this.handleTransmitError(var4);
            return;
         } catch (IllegalStateException var10) {
            this.handleTransmitError(var4);
            LOGGER.log(Level.INFO, "Exception on accessing card ", (Throwable)var10);
            return;
         }

         this.serverCardEventsList.transmit(var6);
      }
   }

   private void handleTransmitError(CardTerminal var1) {
      if (this.cardStateMachine.setCardStatus(SmartCardReaderStatus.ABSENT)) {
         this.serverCardEventsList.cardRemoved();
      } else {
         assert false : "Incorrect card State";
      }

      this.fireCardAccessErrorIfRequired(var1);
   }

   private void fireCardAccessErrorIfRequired(CardTerminal var1) {
   }

   private SmartCardReaderStatus getCardState(CardTerminal var1) {
      try {
         return var1.isCardPresent() ? SmartCardReaderStatus.PRESENT : SmartCardReaderStatus.ABSENT;
      } catch (CardException var3) {
         if (SmartCardClientModule.getCardTerminal(var1.getName()) == null) {
            return SmartCardReaderStatus.ABSENT;
         } else {
            LOGGER.log(Level.INFO, "Exception on determining the cardState ", (Throwable)var3);
            return SmartCardReaderStatus.UNKNOWN;
         }
      }
   }

   private void reset() {
      this.setCardAccessData(null);
   }

   private Card getCard(CardTerminal var1, String var2) throws CardException, InterruptedException, CardResetException, CardNotPresentException {
      Future var3 = this.service.submit(new CardAccessHandler.CardConnectCallable(var1, var2));

      Card var4;
      try {
         var4 = (Card)var3.get();
      } catch (ExecutionException var9) {
         Throwable var5 = var9.getCause();
         if (var5 instanceof CardException) {
            throw (CardException)var5;
         }

         if (var5 instanceof CardResetException) {
            throw (CardResetException)var5;
         }

         throw launderThrowable(var5);
      } finally {
         var3.cancel(true);
      }

      return var4;
   }

   private CardAccessHandler.CardAccessData createCardAccessData(Card var1, int var2, CardTerminal var3) throws InterruptedException, CardResetException {
      Future var4 = this.service.submit(new CardAccessHandler.CardAccessDataCallable(var3, var1, var2));

      CardAccessHandler.CardAccessData var5;
      try {
         var5 = (CardAccessHandler.CardAccessData)var4.get();
      } catch (ExecutionException var10) {
         Throwable var6 = var10.getCause();
         if (var6 instanceof CardResetException) {
            throw (CardResetException)var6;
         }

         if (var6 instanceof InterruptedException) {
            throw (InterruptedException)var6;
         }

         throw launderThrowable(var6);
      } finally {
         var4.cancel(true);
      }

      return var5;
   }

   private byte[] transmitOnWorkerThread(Card var1, byte[] var2, TransportProtoHandler var3) throws CardException, InterruptedException, CardResetException {
      Future var4 = this.service.submit(new CardAccessHandler.TransmitCallable(var1, var3, var2));

      byte[] var5;
      try {
         var5 = (byte[])var4.get();
      } catch (ExecutionException var10) {
         Throwable var6 = var10.getCause();
         if (var6 instanceof CardException) {
            throw (CardException)var6;
         }

         if (var6 instanceof CardResetException) {
            throw (CardResetException)var6;
         }

         if (var6 instanceof InterruptedException) {
            throw (InterruptedException)var6;
         }

         throw launderThrowable(var6);
      } finally {
         var4.cancel(true);
      }

      return var5;
   }

   private synchronized void setCardAccessData(CardAccessHandler.CardAccessData var1) {
      this.cardAccessData = var1;
   }

   private synchronized CardAccessHandler.CardAccessData getCardAccessData() {
      return this.cardAccessData;
   }

   private static RuntimeException launderThrowable(Throwable var0) {
      if (var0 instanceof Error) {
         throw (Error)var0;
      } else {
         return (RuntimeException)(var0 instanceof RuntimeException ? (RuntimeException)var0 : new IllegalStateException("Unexpected Checked Exception ", var0));
      }
   }

   private static boolean isCardReset(Throwable var0) {
      return var0 != null
         && "sun.security.smartcardio.PCSCException".equals(var0.getClass().getName())
         && ("SCARD_W_RESET_CARD".equals(var0.getMessage()) || "SCARD_E_INVALID_VALUE".equals(var0.getMessage()));
   }

   private void waitTillWorkstationUnlocked() throws InterruptedException {
      if (this.workstationUnlockDetector != null) {
         while (!this.workstationUnlockDetector.isWorkstationUnlocked()) {
            Thread.sleep(1000L);
         }
      }
   }

   private static class CardAccessData {
      private final Card card;
      private final CardTerminal terminal;
      private final TransportProtoHandler transportProtoHandler;
      private final byte[] atrBytes;

      public CardAccessData(Card var1, CardTerminal var2, TransportProtoHandler var3, byte[] var4) {
         this.card = var1;
         this.terminal = var2;
         this.transportProtoHandler = var3;
         this.atrBytes = var4;
      }

      public Card getCard() {
         return this.card;
      }

      public CardTerminal getTerminal() {
         return this.terminal;
      }

      public TransportProtoHandler getTransportProtoHandler() {
         return this.transportProtoHandler;
      }

      public byte[] getAtrBytes() {
         return this.atrBytes;
      }
   }

   private class CardAccessDataCallable implements Callable<CardAccessHandler.CardAccessData> {
      private final CardTerminal terminal;
      private final Card c;
      private final int i;

      public CardAccessDataCallable(CardTerminal var2, Card var3, int var4) {
         this.terminal = var2;
         this.c = var3;
         this.i = var4;
      }

      public CardAccessHandler.CardAccessData call() throws Exception {
         Lock var1 = CardAccessHandler.this.lockProvider.getCardLock(this.c);
         var1.lockInterruptibly();
         if (!CardAccessHandler.this.lockProvider.isCardValid(this.c)) {
            var1.unlock();
            throw new CardResetException(null);
         } else {
            CardAccessHandler.CardAccessData var2;
            try {
               var2 = new CardAccessHandler.CardAccessData(
                  this.c,
                  this.terminal,
                  CardAccessHandler.this.transProtoProvider.getTransportProtoHandler(TransportProtoProvider.PROTO[this.i], this.c.getBasicChannel()),
                  this.c.getATR().getBytes()
               );
            } catch (IllegalStateException var6) {
               throw new CardResetException(var6);
            } finally {
               var1.unlock();
            }

            return var2;
         }
      }
   }

   private class CardAccessHandlerThrFactory extends ThreadExecutorThrFactory {
      public CardAccessHandlerThrFactory(Class<?> var2, String var3) {
         super(var2, var3);
      }

      @Override
      public Thread newThread(final Runnable var1) {
         return super.newThread(new Runnable() {
            @Override
            public void run() {
               try {
                  var1.run();
               } finally {
                  Lock var4 = CardAccessHandler.this.thrLocalLock.get();
                  if (var4 != null) {
                     try {
                        CardAccessHandler.this.endTransaction(CardAccessHandler.this.thrLocalCard.get(), var4);
                     } catch (CardException var11) {
                     } catch (CardResetException var12) {
                     }
                  }
               }
            }
         });
      }
   }

   private class CardConnectCallable implements Callable<Card> {
      private final CardTerminal terminal;
      private final String proto;

      public CardConnectCallable(CardTerminal var2, String var3) {
         this.terminal = var2;
         this.proto = var3;
      }

      public Card call() throws Exception {
         try {
            return this.terminal.connect(this.proto);
         } catch (CardException var2) {
            if (CardAccessHandler.isCardReset(var2.getCause())) {
               throw new CardResetException(var2);
            } else {
               throw var2;
            }
         }
      }
   }

   private class TransmitCallable implements Callable<byte[]> {
      private final byte[] command;
      private final Card c;
      private final TransportProtoHandler transportProtoHandler;

      public TransmitCallable(Card var2, TransportProtoHandler var3, byte[] var4) {
         this.command = var4;
         this.c = var2;
         this.transportProtoHandler = var3;
      }

      public byte[] call() throws Exception {
         CommandAPDU var1 = new CommandAPDU(this.command);
         if (var1.getCLA() >= 0 && var1.getINS() == 112) {
            CardAccessHandler.LOGGER.log(Level.INFO, "CAPDU INS " + Integer.toHexString(var1.getINS()));
            ResponseAPDU var2 = null;
            if (var1.getP1() == 0) {
               if (var1.getP2() == 0) {
                  var2 = new ResponseAPDU(new byte[]{106, -127});
               } else {
                  var2 = new ResponseAPDU(new byte[]{106, -122});
               }
            } else if (var1.getP1() < 0) {
               var2 = new ResponseAPDU(new byte[]{110, 0});
            }

            if (var2 != null) {
               CardAccessHandler.LOGGER.log(Level.INFO, "RAPDU STATUS " + Integer.toHexString(var2.getSW1()) + Integer.toHexString(var2.getSW2()));
               return var2.getBytes();
            }
         }

         Lock var10 = CardAccessHandler.this.lockProvider.getCardLock(this.c);
         if (!((ReentrantLock)var10).isHeldByCurrentThread()) {
            var10.lockInterruptibly();
            Lock var3 = CardAccessHandler.this.thrLocalLock.get();
            if (var3 != null) {
               Card var4 = CardAccessHandler.this.thrLocalCard.get();
               CardAccessHandler.this.lockProvider.invalidateCard(var4);
               CardAccessHandler.this.disconnectCard(var4, true);
               CardAccessHandler.this.cleanup(var3);
            }

            CardAccessHandler.this.thrLocalLock.set(var10);
            if (!CardAccessHandler.this.lockProvider.isCardValid(this.c)) {
               CardAccessHandler.this.cleanup(var10);
               throw new CardResetException(null);
            }

            try {
               this.c.beginExclusive();
            } catch (CardException var7) {
               CardAccessHandler.LOGGER.log(Level.INFO, "Exception on beginExclusive ", (Throwable)var7);
               if (CardAccessHandler.isCardReset(var7.getCause())) {
                  CardAccessHandler.this.lockProvider.invalidateCard(this.c);
                  CardAccessHandler.this.disconnectCard(this.c, false);
                  CardAccessHandler.this.cleanup(var10);
                  throw new CardResetException(var7);
               }

               CardAccessHandler.this.lockProvider.invalidateCard(this.c);
               CardAccessHandler.this.disconnectCard(this.c, false);
               CardAccessHandler.this.cleanup(var10);
               throw var7;
            } catch (IllegalStateException var8) {
               CardAccessHandler.this.lockProvider.invalidateCard(this.c);
               CardAccessHandler.this.disconnectCard(this.c, false);
               CardAccessHandler.this.cleanup(var10);
               throw new CardResetException(var8);
            } catch (Exception var9) {
               CardAccessHandler.this.lockProvider.invalidateCard(this.c);
               CardAccessHandler.this.disconnectCard(this.c, false);
               CardAccessHandler.this.cleanup(var10);
               throw var9;
            }

            CardAccessHandler.this.thrLocalCard.set(this.c);
         }

         try {
            return this.transportProtoHandler.transmit(this.command);
         } catch (CardException var5) {
            CardAccessHandler.LOGGER.log(Level.INFO, "Exception on transmit ", (Throwable)var5);
            if (CardAccessHandler.isCardReset(var5.getCause())) {
               CardAccessHandler.this.disconnectCard(this.c, false);
               CardAccessHandler.this.lockProvider.invalidateCard(this.c);
               CardAccessHandler.this.endTransaction(this.c, var10);
               throw new CardResetException(var5);
            } else {
               CardAccessHandler.this.endTransaction(this.c, var10);
               throw var5;
            }
         } catch (Exception var6) {
            CardAccessHandler.this.endTransaction(this.c, var10);
            throw var6;
         }
      }
   }
}
