package com.raritan.smartcard.impl;

import com.raritan.smartcard.SmartCardReaderStatus;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.RejectedExecutionException;
import java.util.logging.Logger;
import javax.smartcardio.CardTerminal;

public class CardEventsProcessor implements SmartCardModuleListener, SmartCardReaderAtomicEventsListener, CardTransmitHandler {
   private final ServerCardEventsListener serverCardEventsList;
   private final SmartCardReaderListener16 cardAccessHandler;
   private final CardTransmitHandler cardTxHandler;
   private final CardAndReaderEventsStateMachine stateMachine;
   private final String tag;
   private final List<Future<Object>> submittedTasks;
   private final ExecutorService service;
   private final EventsListenerManager eventsManager;
   private static final Logger LOGGER = Logger.getLogger(CardEventsProcessor.class.getName());

   public CardEventsProcessor(String var1, SmartCardReaderListener16 var2, ServerCardEventsListener var3, CardTransmitHandler var4, EventsListenerManager var5) {
      this.serverCardEventsList = var3;
      this.cardAccessHandler = var2;
      this.stateMachine = new CardAndReaderEventsStateMachine();
      this.cardTxHandler = var4;
      this.tag = var1;
      this.service = Executors.newSingleThreadExecutor(new ThreadExecutorThrFactory(CardEventsProcessor.class, this.tag));
      this.submittedTasks = new ArrayList<>();
      this.eventsManager = var5;
   }

   @Override
   public synchronized void cardInserted(CardTerminal var1) {
      this.purgeSubmittedTasks();

      try {
         this.submittedTasks.add(this.service.submit(new ExceptionPrintingCallable<>(new CardEventsProcessor.CardInsertedCallable(var1), this.eventsManager)));
      } catch (RejectedExecutionException var3) {
      }
   }

   @Override
   public synchronized void cardReaderInserted(String var1) {
      this.submittedTasks.add(this.service.submit(new ExceptionPrintingCallable<>(new CardEventsProcessor.CardReaderInsertedCallable(), this.eventsManager)));
   }

   @Override
   public synchronized void cardReaderRemoved(String var1) {
      if (!this.service.isShutdown()) {
         this.cancelSubmittedTasks();
         this.purgeSubmittedTasks();
         this.service.submit(new ExceptionPrintingCallable<>(new CardEventsProcessor.CardReaderRemovedCallable(var1), this.eventsManager));
         this.service.shutdown();
      }
   }

   @Override
   public synchronized void cardRemoved(CardTerminal var1) {
      this.cancelSubmittedTasks();
      this.purgeSubmittedTasks();

      try {
         this.submittedTasks.add(this.service.submit(new ExceptionPrintingCallable<>(new CardEventsProcessor.CardRemovedCallable(var1), this.eventsManager)));
      } catch (RejectedExecutionException var3) {
      }
   }

   @Override
   public synchronized void removeAndReinsert(CardTerminal var1) {
      if (!this.service.isShutdown()) {
         this.cancelSubmittedTasks();
         this.purgeSubmittedTasks();
         this.submittedTasks
            .add(this.service.submit(new ExceptionPrintingCallable<>(new CardEventsProcessor.CardRemoveAndReinsertCallable(var1), this.eventsManager)));
      }
   }

   @Override
   public synchronized void transmit(byte[] var1) {
      this.purgeSubmittedTasks();

      try {
         this.submittedTasks.add(this.service.submit(new ExceptionPrintingCallable<>(new CardEventsProcessor.CardTransmitCallable(var1), this.eventsManager)));
      } catch (RejectedExecutionException var3) {
      }
   }

   private void purgeSubmittedTasks() {
      Iterator var1 = this.submittedTasks.iterator();

      while (var1.hasNext()) {
         if (((Future)var1.next()).isDone()) {
            var1.remove();
         }
      }
   }

   private void cancelSubmittedTasks() {
      for (Future var2 : this.submittedTasks) {
         var2.cancel(true);
      }
   }

   private class CardInsertedCallable implements Callable<Object> {
      private final CardTerminal terminal;

      public CardInsertedCallable(CardTerminal var2) {
         this.terminal = var2;
      }

      @Override
      public Object call() {
         if (CardEventsProcessor.this.stateMachine.setCardStatus(SmartCardReaderStatus.PRESENT)) {
            CardEventsProcessor.this.cardAccessHandler.cardInserted(this.terminal);
         }

         return null;
      }
   }

   private class CardReaderInsertedCallable implements Callable<Object> {
      private CardReaderInsertedCallable() {
      }

      @Override
      public Object call() {
         if (CardEventsProcessor.this.stateMachine.setCardReaderStatus(SmartCardReaderStatus.PRESENT)) {
            CardEventsProcessor.this.serverCardEventsList.cardReaderInserted();
         }

         return null;
      }
   }

   private class CardReaderRemovedCallable implements Callable<Object> {
      private final String cardReaderName;

      public CardReaderRemovedCallable(String var2) {
         this.cardReaderName = var2;
      }

      @Override
      public Object call() {
         if (CardEventsProcessor.this.stateMachine.setCardStatus(SmartCardReaderStatus.ABSENT)) {
            CardEventsProcessor.this.cardAccessHandler.cardRemoved(null);
         }

         if (CardEventsProcessor.this.stateMachine.setCardReaderStatus(SmartCardReaderStatus.ABSENT)) {
            CardEventsProcessor.this.cardAccessHandler.cardReaderRemoved(this.cardReaderName);
         }

         return null;
      }
   }

   private class CardRemoveAndReinsertCallable implements Callable<Object> {
      private final CardTerminal terminal;

      public CardRemoveAndReinsertCallable(CardTerminal var2) {
         this.terminal = var2;
      }

      @Override
      public Object call() {
         if (CardEventsProcessor.this.stateMachine.setCardStatus(SmartCardReaderStatus.ABSENT)) {
            CardEventsProcessor.this.cardAccessHandler.cardRemoved(this.terminal);
            if (CardEventsProcessor.this.stateMachine.setCardStatus(SmartCardReaderStatus.PRESENT)) {
               CardEventsProcessor.this.cardAccessHandler.cardInserted(this.terminal);
            } else {
               assert false : "CardStateIncorrect";
            }
         }

         return null;
      }
   }

   private class CardRemovedCallable implements Callable<Object> {
      private final CardTerminal terminal;

      public CardRemovedCallable(CardTerminal var2) {
         this.terminal = var2;
      }

      @Override
      public Object call() {
         if (CardEventsProcessor.this.stateMachine.setCardStatus(SmartCardReaderStatus.ABSENT)) {
            CardEventsProcessor.this.cardAccessHandler.cardRemoved(this.terminal);
         }

         return null;
      }
   }

   private class CardTransmitCallable implements Callable<Object> {
      private final byte[] data;

      public CardTransmitCallable(byte[] var2) {
         this.data = var2;
      }

      @Override
      public Object call() {
         if (SmartCardReaderStatus.PRESENT.equals(CardEventsProcessor.this.stateMachine.getCardState())) {
            CardEventsProcessor.this.cardTxHandler.transmit(this.data);
         }

         return null;
      }
   }
}
