package com.raritan.smartcard.impl;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.RejectedExecutionException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.smartcardio.CardException;
import javax.smartcardio.CardTerminal;
import javax.smartcardio.TerminalFactory;
import nn.pp.core.Platform;

class SmartCardReaderMonitor {
   private final SmartCardReaderListener16 listener;
   private final ExecutorService service = Executors.newCachedThreadPool(new ThreadExecutorThrFactory(SmartCardReaderMonitor.class, null, true));
   private static final int POLL_TIMEOUT = 1000;
   private final TerminalFactory factory;
   private final Map<String, SmartCardReaderMonitor.FutureWrapper> runningTasks = Collections.synchronizedMap(new HashMap<>());
   private static final Logger LOGGER = Logger.getLogger(SmartCardReaderMonitor.class.getName());

   SmartCardReaderMonitor(TerminalFactory var1, SmartCardReaderListener16 var2) {
      this.listener = var2;
      this.factory = var1;
   }

   void monitor(String var1) {
      if (!this.runningTasks.containsKey(var1)) {
         SmartCardReaderMonitor.FutureWrapper var2 = new SmartCardReaderMonitor.FutureWrapper();
         this.runningTasks.put(var1, var2);

         Future var3;
         try {
            var3 = this.service.submit(new ExceptionPrintingCallable<>(new SmartCardReaderMonitor.ReaderMonitorRunnable(var1), null));
         } catch (RejectedExecutionException var5) {
            this.runningTasks.remove(var1);
            throw var5;
         }

         var2.setFuture(var3);
      } else {
         assert false : "There should not be task running for this " + var1;
      }
   }

   void unmonitor(String var1) {
      SmartCardReaderMonitor.FutureWrapper var2 = this.runningTasks.remove(var1);
      if (null != var2) {
         assert var2.getFuture() != null;

         var2.getFuture().cancel(true);
      }
   }

   void dispose() {
      this.service.shutdownNow();
      this.runningTasks.clear();
   }

   private static class FutureWrapper {
      private Future<?> future;

      private FutureWrapper() {
      }

      public synchronized Future<?> getFuture() {
         return this.future;
      }

      public synchronized void setFuture(Future<?> var1) {
         this.future = var1;
      }
   }

   private class ReaderMonitorRunnable implements Callable<Void> {
      private final String cardReaderName;

      public ReaderMonitorRunnable(String var2) {
         this.cardReaderName = var2;
      }

      public Void call() {
         CardTerminal var1 = SmartCardReaderMonitor.this.factory.terminals().getTerminal(this.cardReaderName);
         if (null == var1) {
            SmartCardReaderMonitor.this.runningTasks.remove(this.cardReaderName);
            SmartCardReaderMonitor.this.listener.cardReaderRemoved(this.cardReaderName);
         } else {
            try {
               while (this.waitForCardPresent(var1)) {
                  SmartCardReaderMonitor.this.listener.cardInserted(var1);
                  if (!this.waitForCardAbsent(var1)) {
                     break;
                  }

                  SmartCardReaderMonitor.this.listener.cardRemoved(var1);
               }
            } catch (Exception var3) {
               SmartCardReaderMonitor.LOGGER.log(Level.INFO, "Exception happenned on monitoring card", (Throwable)var3);
               SmartCardReaderMonitor.this.runningTasks.remove(this.cardReaderName);
               SmartCardReaderMonitor.this.listener.cardReaderRemoved(this.cardReaderName);
            }
         }

         return null;
      }

      private boolean waitForCardPresent(CardTerminal var1) throws CardException {
         if (!Platform.isLinux()) {
            while (!var1.waitForCardPresent(1000L)) {
               if (Thread.currentThread().isInterrupted()) {
                  return false;
               }
            }

            return true;
         } else {
            while (!var1.isCardPresent()) {
               try {
                  Thread.sleep(1000L);
               } catch (InterruptedException var3) {
                  Thread.currentThread().interrupt();
                  return false;
               }
            }

            return true;
         }
      }

      private boolean waitForCardAbsent(CardTerminal var1) throws CardException {
         if (!Platform.isLinux()) {
            do {
               try {
                  if (var1.waitForCardAbsent(1000L)) {
                     return true;
                  }
               } catch (CardException var4) {
                  if (Platform.isWindows() && !"wait mismatch".equalsIgnoreCase(var4.getMessage())) {
                     throw var4;
                  }
               }
            } while (!Thread.currentThread().isInterrupted());

            return false;
         } else {
            while (var1.isCardPresent()) {
               try {
                  Thread.sleep(1000L);
               } catch (InterruptedException var3) {
                  Thread.currentThread().interrupt();
                  return false;
               }
            }

            return true;
         }
      }
   }
}
