package com.raritan.smartcard.impl;

import com.raritan.smartcard.CardAccessErrorsListener;
import com.raritan.smartcard.SmartCardException;
import com.raritan.smartcard.SmartCardReaderListener;
import com.raritan.smartcard.SmartCardReaderSession;
import com.raritan.smartcard.SmartCardSessionEventsListener;
import com.raritan.smartcard.impl.crp.CRPHandler;
import com.raritan.smartcard.impl.crp.ServerCardEventsListenerImpl;
import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.atomic.AtomicReference;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.smartcardio.CardTerminal;
import nn.pp.core.INotificationEvent;
import nn.pp.core.NotificationListener;
import nn.pp.core.WorkstationUnlockDetector;
import nn.pp.core.impl.DeviceConnector;
import nn.pp.core.impl.NotificationListenerList;

public class SmartCardReaderSessionImpl implements SmartCardReaderSession {
   private static final int CLEANUP_WAIT_TIME = 60000;
   private final String cardReaderName;
   private final String host;
   private final int port;
   private final boolean sslMode;
   private final int rfbSessionId;
   private final String rdmSessionID;
   private final String ericKey;
   private final String tag;
   private final CardAccessManager cardAccessManager;
   private final CardStateTrackerListenerList cardStateListenerList;
   private final CardAccessErrorsListenerList cardAccessErrorsListenerList;
   private final NotificationListenerList notificationListenerList;
   private final SmartCardSessionEventsListenerList smartCardSessionEventsListenerList;
   private final SmartCardReaderSessionImpl.Scrl scrl;
   private final SmartCardReaderSessionImpl.Cael cael;
   private final CRPHandler protoHandler;
   private final AtomicReference<Object> handleRef = new AtomicReference<>();
   private final AtomicReference<Enum<SmartCardReaderSessionImpl.State>> state = new AtomicReference<>(SmartCardReaderSessionImpl.State.NOT_STARTED);
   private static final Logger LOGGER = Logger.getLogger(SmartCardReaderSessionImpl.class.getName());
   private final int msindex;
   private volatile DeviceConnector deviceConnector;

   public SmartCardReaderSessionImpl(
      String var1, String var2, int var3, boolean var4, int var5, int var6, String var7, String var8, String var9, CardAccessManager var10
   ) {
      this.cardReaderName = var1;
      this.host = var2;
      this.port = var3;
      this.sslMode = var4;
      this.rfbSessionId = var5;
      this.msindex = var6;
      this.rdmSessionID = var7;
      this.ericKey = var8;
      this.tag = var9;
      this.cardAccessManager = var10;
      this.scrl = new SmartCardReaderSessionImpl.Scrl();
      this.cardStateListenerList = new CardStateTrackerListenerList();
      this.cael = new SmartCardReaderSessionImpl.Cael();
      this.cardAccessErrorsListenerList = new CardAccessErrorsListenerList();
      this.notificationListenerList = new NotificationListenerList();
      this.smartCardSessionEventsListenerList = new SmartCardSessionEventsListenerList();
      this.protoHandler = new CRPHandler();
   }

   @Override
   public void close() {
      Enum var1 = this.state.getAndSet(SmartCardReaderSessionImpl.State.CLOSE_CALLED);
      if (var1 == SmartCardReaderSessionImpl.State.START_CALLED) {
         this.close(true);
      }
   }

   private void close(boolean var1) {
      Object var2 = this.handleRef.get();
      if (!(var2 instanceof Boolean)) {
         CardAccessManager.Handle var3 = (CardAccessManager.Handle)this.handleRef.get();
         if (var3 != null || !this.handleRef.compareAndSet(null, Boolean.TRUE)) {
            var3 = (CardAccessManager.Handle)this.handleRef.get();
            this.cardAccessManager.unmountSmartCardReader(var3);
            if (var1) {
               this.startCleanupTimer();
            }
         }
      }
   }

   private void startCleanupTimer() {
      Timer var1 = new Timer("Cleanup Timer for " + this.cardReaderName + "-" + this.tag, true);
      var1.schedule(new SmartCardReaderSessionImpl.CleanupTask(var1), 60000L);
   }

   @Override
   public void start(
      SmartCardReaderListener var1,
      CardAccessErrorsListener var2,
      NotificationListener var3,
      SmartCardSessionEventsListener var4,
      WorkstationUnlockDetector var5
   ) throws SmartCardException, IOException {
      if (this.state.compareAndSet(SmartCardReaderSessionImpl.State.NOT_STARTED, SmartCardReaderSessionImpl.State.START_CALLED)) {
         DeviceConnector var6 = new DeviceConnector(Logger.getLogger("com.raritan.smartcard"), null);

         try {
            var6.connect(this.host, this.port, this.sslMode);
         } catch (IOException var11) {
            throw var11;
         }

         this.deviceConnector = var6;

         try {
            this.protoHandler.init(var6, Logger.getLogger("com.raritan.smartcard"));
         } catch (IOException var10) {
            var6.disconnect();
            throw var10;
         }

         this.protoHandler.addListener(SmartCardSessionEventsListener.class, new SmartCardReaderSessionImpl.Scsel());
         this.protoHandler.addListener(NotificationListener.class, new SmartCardReaderSessionImpl.Nl());
         if (var3 != null) {
            this.notificationListenerList.addListener(var3);
         }

         if (var4 != null) {
            this.smartCardSessionEventsListenerList.addListener(var4);
         }

         try {
            this.protoHandler.connect(null, null, null, this.ericKey, this.rdmSessionID);
         } catch (IOException var8) {
            var6.disconnect();
            throw var8;
         } catch (SmartCardException var9) {
            var6.disconnect();
            throw var9;
         }

         if (var1 != null) {
            this.cardStateListenerList.addListener(var1);
         }

         if (var2 != null) {
            this.cardAccessErrorsListenerList.addListener(var2);
         }

         CardAccessManager.Handle var7 = this.cardAccessManager
            .mountSmartCardReader(
               this.cardReaderName,
               this.tag,
               new ServerCardEventsListenerImpl(this.protoHandler, this.rfbSessionId, this.msindex, this.protoHandler),
               this.scrl,
               this.cael,
               this.protoHandler,
               var5
            );
         if (!this.handleRef.compareAndSet(null, var7)) {
            LOGGER.log(Level.INFO, "Cleaning up");
            this.cardAccessManager.unmountSmartCardReader(var7);
            this.startCleanupTimer();
         }
      }
   }

   @Override
   public void addSmartCardReaderListener(SmartCardReaderListener var1) {
      if (var1 != null) {
         this.cardStateListenerList.addListener(var1);
      }
   }

   @Override
   public void removeSmartCardReaderListener(SmartCardReaderListener var1) {
      this.cardStateListenerList.removeListener(var1);
   }

   @Override
   public void removeCardAccessErrorsListener(CardAccessErrorsListener var1) {
      this.cardAccessErrorsListenerList.removeListener(var1);
   }

   @Override
   public void removeNotificationListener(NotificationListener var1) {
      this.notificationListenerList.removeListener(var1);
   }

   @Override
   public void removeSmartCardSessionEventsListener(SmartCardSessionEventsListener var1) {
      this.smartCardSessionEventsListenerList.removeListener(var1);
   }

   @Override
   public void simulateRemoveAndReinsert() {
      Object var1 = this.handleRef.get();
      if (var1 instanceof CardAccessManager.Handle) {
         this.cardAccessManager.simulateRemoveAndReinsert((CardAccessManager.Handle)var1);
      }
   }

   private class Cael implements CardAccessErrorsListener {
      private Cael() {
      }

      @Override
      public void errorOnAccessingCard(String var1) {
         SmartCardReaderSessionImpl.this.cardAccessErrorsListenerList.fireErrorOnAccessingCard(var1);
      }

      @Override
      public void noProtocolSupported(String var1) {
         SmartCardReaderSessionImpl.this.cardAccessErrorsListenerList.fireNoProtocolSupported(var1);
      }
   }

   private class CleanupTask extends TimerTask {
      private final Timer cleanupTimer;

      public CleanupTask(Timer var2) {
         this.cleanupTimer = var2;
      }

      @Override
      public void run() {
         SmartCardReaderSessionImpl.LOGGER.log(Level.INFO, "Cleaning up");
         SmartCardReaderSessionImpl.this.deviceConnector.disconnect();
         this.cleanupTimer.cancel();
      }
   }

   private class Nl implements NotificationListener {
      private Nl() {
      }

      @Override
      public void receivedNotification(INotificationEvent var1) {
         if (var1.isQuit() || var1.isError()) {
            SmartCardReaderSessionImpl.LOGGER.log(Level.INFO, "Trace of Quit", (Throwable)(new Exception("TRACE FOR DEBUG")));
            SmartCardReaderSessionImpl.LOGGER.log(Level.INFO, "Reason for Quit - " + var1.getMessage());
            SmartCardReaderSessionImpl.this.close(false);
            SmartCardReaderSessionImpl.this.protoHandler.close();
            SmartCardReaderSessionImpl.this.deviceConnector.disconnect();
         }

         SmartCardReaderSessionImpl.this.notificationListenerList.fireNotification(var1);
      }

      @Override
      public void textNotification(String var1) {
      }
   }

   private class Scrl implements SmartCardReaderListener16 {
      private Scrl() {
      }

      @Override
      public void cardInserted(CardTerminal var1) {
         SmartCardReaderSessionImpl.this.cardStateListenerList.fireCardInserted(SmartCardReaderSessionImpl.this.cardReaderName);
      }

      @Override
      public void cardReaderRemoved(String var1) {
         SmartCardReaderSessionImpl.this.cardStateListenerList.fireCardReaderRemoved(var1);
      }

      @Override
      public void cardRemoved(CardTerminal var1) {
         SmartCardReaderSessionImpl.this.cardStateListenerList.fireCardRemoved(SmartCardReaderSessionImpl.this.cardReaderName);
      }
   }

   private class Scsel implements SmartCardSessionEventsListener {
      private Scsel() {
      }

      @Override
      public void cardReaderMounted(String var1) {
         SmartCardReaderSessionImpl.this.smartCardSessionEventsListenerList.firecardReaderMounted(var1);
      }

      @Override
      public void disconnected(Exception var1) {
         SmartCardReaderSessionImpl.LOGGER.log(Level.INFO, "disconnect received", (Throwable)var1);
         SmartCardReaderSessionImpl.this.close(false);
         SmartCardReaderSessionImpl.this.protoHandler.close();
         SmartCardReaderSessionImpl.this.deviceConnector.disconnect();
         SmartCardReaderSessionImpl.this.smartCardSessionEventsListenerList.fireDisconnected(var1);
      }
   }

   private static enum State {
      NOT_STARTED,
      START_CALLED,
      CLOSE_CALLED;
   }
}
