package com.raritan.smartcard.impl.crp;

import com.raritan.smartcard.SmartCardException;
import com.raritan.smartcard.SmartCardIncompatibleProtoException;
import com.raritan.smartcard.SmartCardSessionEventsListener;
import com.raritan.smartcard.impl.EventsListenerManager;
import com.raritan.smartcard.impl.commands.AuthHttpIDGetChallenge;
import com.raritan.smartcard.impl.commands.AuthRDMRequest;
import com.raritan.smartcard.impl.commands.ClientCommandsExecutor;
import com.raritan.smartcard.impl.commands.SupportsVisitor;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.EventListener;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.event.EventListenerList;
import nn.pp.core.impl.ProtocolHandler;

public class CRPHandler extends ProtocolHandler<SmartCardException> implements ClientCommandsExecutor, EventsListenerManager {
   private volatile CRPProtoHandler<?> protoHandler;
   private final EventListenerList listenersList = new EventListenerList();
   private static final Logger LOGGER = Logger.getLogger(CRPHandler.class.getName());

   public CRPHandler() {
      this.versionMajor = -1;
      this.versionMinor = -1;
   }

   @Override
   protected void disconnected(Exception var1) {
      this.protoHandler.handleDisconnection();

      for (SmartCardSessionEventsListener var3 : this.getListeners(SmartCardSessionEventsListener.class)) {
         var3.disconnected(var1);
      }
   }

   @Override
   protected String getProtocolName() {
      return "CRP";
   }

   protected SmartCardException loadException(String var1) {
      return null;
   }

   @Override
   protected void loadPdus() {
   }

   @Override
   protected void processInitialHandshake(int var1) throws IOException, SmartCardException {
   }

   @Override
   protected void negotiateProtocolVersion() throws IOException, SmartCardException {
      new CRPHelloMessage().writeMessage(this.os);
      CRPVersionMessage var1 = new CRPVersionMessage().readMessage(this.is);
      this.loadProtocolHandler(var1);
      new CRPVersionAcceptMessage(var1.getVersionMajor(), var1.getVersionMinor()).writeMessage(this.os);
      this.initialHandshakeFinished();
      this.protoHandler.setClientCommandsExecutor(this);
      this.protoHandler.setEventsListenerManager(this);
   }

   @Override
   protected void processProtocol() throws IOException, SmartCardException {
      this.sendAuthenticationMessage();

      while (this.shouldRun) {
         this.handleMessage(this.protoHandler);
      }
   }

   private void sendAuthenticationMessage() throws IOException, SmartCardException {
      if (this.rdmSessionID != null) {
         this.executeClientCommands(new AuthRDMRequest(this.rdmSessionID));
      } else {
         if (this.httpSessionID == null) {
            throw new SmartCardException("No Authentication information available");
         }

         this.executeClientCommands(new AuthHttpIDGetChallenge(this.httpSessionID));
      }
   }

   private <V> void handleMessage(CRPProtoHandler<V> var1) throws IOException, SmartCardException {
      this.executeCommand(var1, var1.getMessageToCommandProducer(this.readServerMessageType()));
   }

   private <V> void executeCommand(CRPProtoHandler<V> var1, MessageToCommandProducer<V> var2) throws IOException, SmartCardException {
      if (var2 == null) {
         throw new SmartCardException("Protocol Error: Unknown Protocol message received");
      } else {
         var2 = var2.readMessage(this.is);
         LOGGER.log(Level.INFO, "Received Command " + var2);
         var1.executeCommand(var2);
      }
   }

   private void loadProtocolHandler(CRPVersionMessage var1) throws SmartCardException {
      String var2 = CRPHandler.class.getPackage().getName() + "." + var1.getVersionAsString() + "." + "CRPProtoHandler" + var1.getVersionAsString();
      LOGGER.log(Level.INFO, "Loading CRP Protocol Handler impl class with name - " + var2);

      Class var3;
      try {
         var3 = Class.forName(var2);
      } catch (ClassNotFoundException var14) {
         throw new SmartCardIncompatibleProtoException(var14);
      }

      Class var4;
      try {
         var4 = var3.asSubclass(CRPProtoHandler.class);
      } catch (ClassCastException var13) {
         throw new SmartCardException(var13);
      }

      Constructor var5 = null;

      try {
         var5 = var4.getConstructor();
      } catch (SecurityException var11) {
         throw new SmartCardException(var11);
      } catch (NoSuchMethodException var12) {
         throw new SmartCardException(var12);
      }

      try {
         this.protoHandler = (CRPProtoHandler<?>)var5.newInstance();
      } catch (IllegalArgumentException var7) {
         throw new SmartCardException(var7);
      } catch (InstantiationException var8) {
         throw new SmartCardException(var8);
      } catch (IllegalAccessException var9) {
         throw new SmartCardException(var9);
      } catch (InvocationTargetException var10) {
         throw new SmartCardException(var10);
      }
   }

   @Override
   public <T> void executeClientCommands(SupportsVisitor<T> var1) {
      LOGGER.log(Level.INFO, "Sending command for execution - " + var1);

      try {
         this.protoHandler.executeClientCommands(this.os, this.is, var1);
      } catch (IOException var3) {
         this.close();
         this.disconnected(var3);
      } catch (SmartCardException var4) {
         this.close();
         this.disconnected(var4);
      } catch (RuntimeException var5) {
         this.close();
         this.disconnected(var5);
         throw var5;
      }
   }

   @Override
   public <T extends EventListener> void addListener(Class<T> var1, T var2) {
      this.listenersList.add(var1, var2);
   }

   @Override
   public <T extends EventListener> void removeListener(Class<T> var1, T var2) {
      this.listenersList.remove(var1, var2);
   }

   @Override
   public <T extends EventListener> List<T> getListeners(Class<T> var1) {
      return Arrays.asList((T[])this.listenersList.getListeners(var1));
   }
}
