package nn.pp.audiocore.impl.audio;

import java.io.IOException;
import java.util.Timer;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.TargetDataLine;
import javax.sound.sampled.LineEvent.Type;
import javax.sound.sampled.Mixer.Info;
import nn.pp.audiocore.AudioException;
import nn.pp.audiocore.AudioFormat;
import nn.pp.audiocore.impl.ThreadMonitor;
import nn.pp.core.T;

public class Capture extends Thread {
   private static final boolean OVERRIDE_DURATION = true;
   private static final int RETRY_TIMEOUT = 1000;
   private static final int DEFAULT_BLOCK_DURATION_CAPTURE = 40;
   private boolean shouldRun = false;
   private ThreadMonitor captureMonitor;
   private ThreadMonitor consumeMonitor;
   private Thread consumeThread;
   private CircularBufferStream circularStream;
   private static final boolean TRACE_BUFFERS = false;
   private int overruns = 0;
   private Timer traceTimer;
   private int traceCount = 0;
   private TargetDataLine targetLine;
   private LineListener targetListener;
   private Info mixer;
   private AudioFormat format;
   private Logger logger;
   private byte[] readBuffer = null;
   private byte[] consumeBuffer = null;
   private int blockDuration = 40;
   private int packetSize;
   private AudioConsumer consumer;

   public Capture(Logger var1, Info var2, AudioFormat var3, AudioConsumer var4) {
      this.logger = var1;
      this.mixer = var2;
      this.format = var3;
      this.consumer = var4;
      this.circularStream = new CircularBufferStream(var3.getBytesPerSecond());
      this.captureMonitor = new ThreadMonitor(0);
      this.consumeMonitor = new ThreadMonitor(0);
      this.consumeThread = new Thread() {
         @Override
         public void run() {
            Capture.this.consumeCapture();
         }
      };
   }

   public void activate() {
      this.packetSize = this.format.getBytesPerSecond() * this.blockDuration / 1000;
      if (!this.shouldRun) {
         this.shouldRun = true;
         this.start();
         this.consumeThread.start();
      }
   }

   public void close() {
      this.shouldRun = false;
      this.wakeupCapture();
      this.wakeupConsumer();

      try {
         this.join(1000L);
         if (this.isAlive()) {
            this.logger.log(Level.WARNING, T._("Capture thread still alive, killing forcefully!"));
            this.stop();
         }
      } catch (InterruptedException var3) {
      }

      try {
         this.consumeThread.join(1000L);
         if (this.consumeThread.isAlive()) {
            this.logger.log(Level.WARNING, T._("Consumer thread still alive, killing forcefully!"));
            this.consumeThread.stop();
         }
      } catch (InterruptedException var2) {
      }

      this.readBuffer = null;
      this.consumeBuffer = null;
   }

   public void setBlockDuration(int var1) {
   }

   public void pauseCapture() {
      synchronized (this.captureMonitor) {
         this.captureMonitor.setValue(0);
      }
   }

   public void resumeCapture() {
      this.wakeupCapture();
   }

   private void checkConsumeBufSize(int var1) {
      if (this.consumeBuffer == null || this.consumeBuffer.length < var1) {
         this.consumeBuffer = new byte[var1];
      }
   }

   private void deactivate() throws IOException {
      this.logger.log(Level.INFO, T._("cleaning up targetLine"));
      if (this.targetLine != null) {
         this.targetLine.flush();
         this.targetLine.removeLineListener(this.targetListener);
         this.targetLine.stop();
         this.targetLine.close();
         this.targetLine = null;
      }
   }

   private void initCapture() throws AudioException {
      this.logger.log(Level.INFO, T._("trying to acquire a TargetDataLine"));

      try {
         this.targetLine = AudioSystem.getTargetDataLine(this.format, this.mixer);
         this.targetLine.open(this.format, this.format.getBytesPerSecond());
      } catch (Exception var2) {
         throw new AudioException(T._("failure during targetline aquisition"));
      }

      this.logger.log(Level.INFO, T._("TargetDataLine has been acquired"));
      this.targetListener = new LineListener() {
         @Override
         public void update(LineEvent var1) {
            if (var1.getType() == Type.START) {
               Capture.this.logger.log(Level.INFO, T._("target line start event"));
            } else if (var1.getType() == Type.STOP) {
               Capture.this.logger.log(Level.INFO, T._("target line stop event"));
            } else if (var1.getType() == Type.OPEN) {
               Capture.this.logger.log(Level.INFO, T._("target line open event"));
            } else if (var1.getType() == Type.CLOSE) {
               Capture.this.logger.log(Level.INFO, T._("target line close event"));
            }
         }
      };
      this.targetLine.addLineListener(this.targetListener);
      this.targetLine.start();
      this.logger.log(Level.INFO, T._("capture interface initialized"));
   }

   private void wakeupCapture() {
      synchronized (this.captureMonitor) {
         this.captureMonitor.setValue(1);
         this.captureMonitor.notify();
      }
   }

   private void wakeupConsumer() {
      synchronized (this.consumeMonitor) {
         this.consumeMonitor.setValue(1);
         this.consumeMonitor.notify();
      }
   }

   @Override
   public void run() {
      this.logger.log(Level.INFO, T._("capture thread started"));

      try {
         this.processCapture();
      } catch (Exception var3) {
         this.logger.log(Level.SEVERE, T._("AudioHandler: processCapture exception"));
         var3.printStackTrace();
      }

      try {
         this.deactivate();
      } catch (Exception var2) {
      }

      this.logger.log(Level.INFO, T._("capture thread finished"));
   }

   private void processCapture() throws IOException, AudioException {
      byte var1 = 10;
      int var2 = 0;
      boolean var3 = false;
      int var4 = this.format.getBytesPerSecond() * var1 / 1000;
      int var5 = this.format.getFrameSize();
      var4 = var4 / var5 * var5;
      this.readBuffer = new byte[var4];

      while (this.shouldRun) {
         while (this.targetLine == null && this.shouldRun) {
            try {
               this.initCapture();
            } catch (AudioException var16) {
               this.logger.log(Level.WARNING, T._("targetline currently unavailable"));

               try {
                  Thread.sleep(1000L);
               } catch (Exception var15) {
               }
            }
         }

         synchronized (this.captureMonitor) {
            while (this.captureMonitor.getValue() == 0) {
               this.logger.log(Level.INFO, "Capture is paused");
               this.deactivate();
               var3 = false;

               try {
                  this.captureMonitor.wait();
               } catch (Exception var14) {
               }
            }

            if (!this.shouldRun) {
               break;
            }
         }

         if (!var3) {
            this.logger.log(Level.INFO, "Capture resumed");
            synchronized (this.circularStream) {
               this.circularStream.bufReset();
            }

            this.initCapture();
            var3 = true;
         }

         if (this.targetLine.available() < var4) {
            try {
               Thread.sleep(var1 / 2);
            } catch (InterruptedException var12) {
            }
         } else {
            try {
               var2 = this.targetLine.read(this.readBuffer, 0, var4);
            } catch (Exception var11) {
               throw new AudioException(T._("targetLine read failed"));
            }

            if (var2 != 0) {
               if (var2 > 0) {
                  synchronized (this.circularStream) {
                     if (var2 > this.circularStream.getWritableBytes()) {
                        this.logger.log(Level.INFO, T._("Buffer overrun detected, expect choppyness"));
                        this.trace("Buffer overrun detected, expect choppyness");
                        this.overruns++;
                     }

                     this.circularStream.write(this.readBuffer, 0, var2);
                  }

                  this.wakeupConsumer();
               } else {
                  this.deactivate();
               }
            }
         }
      }
   }

   private void consumeCapture() {
      while (this.shouldRun) {
         synchronized (this.consumeMonitor) {
            while (this.consumeMonitor.getValue() == 0) {
               try {
                  this.consumeMonitor.wait();
               } catch (Exception var7) {
               }
            }

            this.consumeMonitor.setValue(0);
            if (!this.shouldRun) {
               break;
            }
         }

         int var12;
         synchronized (this) {
            var12 = this.packetSize;
         }

         int var2;
         synchronized (this.circularStream) {
            var2 = this.circularStream.getAvailableBytes();
            if (var2 < var12) {
               continue;
            }

            try {
               this.checkConsumeBufSize(var12);
               int var4 = this.circularStream.read(this.consumeBuffer, var12);

               assert var12 == var4;
            } catch (IOException var9) {
               this.logger.log(Level.SEVERE, T._("Exception handled when reading capture data from buffer"), (Throwable)var9);
               break;
            }
         }

         try {
            this.consumer.consumeAudioData(this.format, this.consumeBuffer, 0, var12);
         } catch (IOException var8) {
            this.logger.log(Level.SEVERE, T._("Exception handled when sending capture data to network"), (Throwable)var8);
            break;
         }

         if (var2 - var12 >= var12) {
            this.wakeupConsumer();
         }
      }

      this.logger.log(Level.INFO, T._("capture consumer thread finished"));
   }

   private void traceBuffers() {
      this.traceCount++;
      int var1;
      int var2;
      synchronized (this.circularStream) {
         var1 = this.circularStream.getAvailableBytes();
         var2 = var1 * 1000 / this.format.getBytesPerSecond();
      }

      this.trace("Capture: " + this.traceCount + " -> Overruns: " + this.overruns + ", buffer: " + var1 + " bytes, " + var2 + " ms");
   }

   private void trace(String var1) {
   }
}
