package nn.pp.audiocore.impl.audio;

import java.io.IOException;
import java.util.Timer;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.LineEvent.Type;
import javax.sound.sampled.Mixer.Info;
import nn.pp.audiocore.AudioException;
import nn.pp.audiocore.AudioFormat;
import nn.pp.audiocore.impl.ThreadMonitor;
import nn.pp.core.Platform;
import nn.pp.core.T;

public class Playback extends Thread implements AudioConsumer {
   private static final int RETRY_TIMEOUT = 1000;
   private static final int DEFAULT_BLOCK_DURATION_PLAYBACK = 120;
   private boolean shouldRun = false;
   private ThreadMonitor monitor;
   private static final boolean TRACE_BUFFERS = false;
   private int overruns = 0;
   private int underruns = 0;
   private Timer traceTimer;
   private int traceCount = 0;
   private Info mixer;
   private AudioFormat format;
   private CircularBufferStream circularStream;
   private SourceDataLine sourceLine;
   private LineListener sourceListener;
   private Logger logger;
   private byte[] writeBuffer = null;
   private int blockDuration = 120;

   public Playback(Logger var1, Info var2, AudioFormat var3) {
      this.logger = var1;
      this.mixer = var2;
      this.format = var3;
      this.monitor = new ThreadMonitor(0);
      this.circularStream = new CircularBufferStream(var3.getBytesPerSecond());
   }

   public void activate() {
      if (!this.shouldRun) {
         this.shouldRun = true;
         this.start();
      }
   }

   public void close() {
      this.shouldRun = false;
      this.wakeup();

      try {
         this.join(1000L);
         if (this.isAlive()) {
            this.logger.log(Level.WARNING, T._("Playback thread still alive, killing forcefully!"));
            this.stop();
         }
      } catch (InterruptedException var2) {
      }

      if (this.traceTimer != null) {
         this.traceTimer.cancel();
         this.traceTimer = null;
      }
   }

   public void setBlockDuration(int var1) {
      synchronized (this) {
         this.blockDuration = var1;
      }
   }

   @Override
   public void consumeAudioData(AudioFormat var1, byte[] var2, int var3, int var4) throws IOException {
      if (this.format.matches(var1)) {
         int var5;
         synchronized (this.circularStream) {
            if (var4 > this.circularStream.getWritableBytes()) {
               this.logger.log(Level.INFO, T._("Buffer overrun detected, expect choppyness"));
               this.trace("Buffer overrun detected, expect choppyness");
               this.overruns++;
            }

            this.circularStream.write(var2, var3, var4);
            var5 = this.circularStream.getAvailableBytes();
         }

         this.logger.log(Level.FINER, "Received " + var4 + " bytes");
         int var11;
         synchronized (this) {
            var11 = this.blockDuration;
         }

         int var7 = var1.getBytesPerSecond();
         if (var5 > var7 * var11 / 1000 || var4 == 0) {
            this.wakeup();
         }
      }
   }

   public void flushBuffers() {
      synchronized (this.circularStream) {
         this.circularStream.bufReset();
      }
   }

   private void reset() {
      this.logger.log(Level.INFO, T._("cleaning up sourceLine"));
      if (this.sourceLine != null) {
         this.sourceLine.flush();
         this.sourceLine.stop();
         this.sourceLine.removeLineListener(this.sourceListener);
         this.sourceLine.close();
         this.sourceLine = null;
      }
   }

   private void checkWriteBufSize(int var1) {
      if (this.writeBuffer == null || this.writeBuffer.length < var1) {
         this.writeBuffer = new byte[var1];
      }
   }

   private void initPlayback() throws AudioException {
      this.logger.log(Level.INFO, T._("trying to acquire a SourceDataLine"));

      try {
         this.sourceLine = AudioSystem.getSourceDataLine(this.format, this.mixer);
         this.sourceLine.open(this.format, this.format.getBytesPerSecond());
      } catch (LineUnavailableException var2) {
         throw new AudioException(T._("failure during sourceline aquisition"));
      }

      this.logger.log(Level.INFO, T._("SourceDataLine has been acquired"));
      this.sourceListener = new LineListener() {
         @Override
         public void update(LineEvent var1) {
            if (var1.getType() == Type.START) {
               Playback.this.logger.log(Level.INFO, T._("source line start event"));
            } else if (var1.getType() == Type.STOP) {
               Playback.this.logger.log(Level.INFO, T._("source line stop event"));
            } else if (var1.getType() == Type.OPEN) {
               Playback.this.logger.log(Level.INFO, T._("source line open event"));
            } else if (var1.getType() == Type.CLOSE) {
               Playback.this.logger.log(Level.INFO, T._("source line close event"));
            }
         }
      };
      this.sourceLine.addLineListener(this.sourceListener);
      this.sourceLine.start();
      this.logger.log(Level.INFO, T._("playback interface initialized"));
   }

   private void wakeup() {
      synchronized (this.monitor) {
         this.monitor.setValue(1);
         this.monitor.notify();
      }
   }

   @Override
   public void run() {
      this.logger.log(Level.INFO, T._("playback thread started"));

      try {
         this.processPlayback();
      } catch (Exception var3) {
         this.logger.log(Level.SEVERE, T._("AudioHandler: processPlayback exception"));
         var3.printStackTrace();
      }

      try {
         this.reset();
      } catch (Exception var2) {
      }

      this.logger.log(Level.INFO, T._("playback thread finished"));
   }

   protected void processPlayback() throws IOException, AudioException {
      int var1 = 0;
      int var2 = 0;
      int var3 = 0;
      boolean var4 = false;
      Mixer var5 = AudioSystem.getMixer(this.mixer);
      if (var5 != null && var5.getClass().getName().equals("com.sun.media.sound.DirectAudioDevice") && Platform.isWindows()) {
         var4 = true;
      }

      while (this.shouldRun) {
         while (this.sourceLine == null && this.shouldRun) {
            try {
               this.initPlayback();
            } catch (AudioException var13) {
               this.logger.log(Level.WARNING, T._("line currently unavailable"));

               try {
                  Thread.sleep(1000L);
               } catch (Exception var12) {
               }
            }

            synchronized (this.circularStream) {
               this.circularStream.bufReset();
            }
         }

         int var21 = this.format.getFrameSize();
         synchronized (this.circularStream) {
            var1 = this.circularStream.getAvailableBytes() / var21 * var21;
            this.checkWriteBufSize(var1);
            var2 = this.circularStream.read(this.writeBuffer, var1);
         }

         if (var2 > 0) {
            try {
               var3 = this.sourceLine.write(this.writeBuffer, 0, var2);
            } catch (Exception var15) {
               throw new IOException(T._("sourceLine transfer error"));
            }
         } else {
            var3 = 0;
         }

         if (var3 < 0) {
            this.logger.log(Level.INFO, T._("bogus sourceline state"));
            this.reset();
         } else if (var2 == 0) {
            boolean var22 = false;
            synchronized (this.monitor) {
               while (this.monitor.getValue() == 0) {
                  if (!var22 && this.sourceLine.isActive() && this.sourceLine.available() == this.sourceLine.getBufferSize()) {
                     this.logger.log(Level.INFO, T._("Buffer underrun detected, re-syncing ..."));
                     this.trace("Buffer underrun detected, re-syncing ...");
                     this.underruns++;
                     if (var4) {
                        this.sourceLine.stop();
                        this.sourceLine.flush();
                     }

                     var22 = true;
                  }

                  try {
                     this.monitor.wait(100L);
                  } catch (Exception var14) {
                  }
               }

               if (var22) {
                  this.logger.log(Level.INFO, T._("... re-syncing done"));
                  this.trace("... re-syncing done");
                  if (var4 && !this.sourceLine.isActive()) {
                     this.sourceLine.start();
                  }

                  var22 = false;
               }

               this.monitor.setValue(0);
            }
         } else {
            this.logger.log(Level.FINER, "Played " + var2 + " bytes of data");
         }
      }
   }

   private void traceBuffers() {
      this.traceCount++;
      int var1;
      int var2;
      synchronized (this.circularStream) {
         var1 = this.circularStream.getAvailableBytes();
         var2 = var1 * 1000 / this.format.getBytesPerSecond();
      }

      this.trace(
         "Playback: " + this.traceCount + " -> Overruns: " + this.overruns + ", underruns: " + this.underruns + ", buffer: " + var1 + " bytes, " + var2 + " ms"
      );
   }

   private void trace(String var1) {
   }
}
