package nn.pp.common.audio;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;
import nn.pp.audiocore.AudioAdapter;
import nn.pp.audiocore.AudioCore;
import nn.pp.audiocore.AudioDevice;
import nn.pp.audiocore.AudioException;
import nn.pp.audiocore.AudioFormat;
import nn.pp.audiocore.impl.audio.DeviceEnumerator;
import nn.pp.common.ui.helpers.SpringUtilities;
import nn.pp.core.INotificationEvent;
import nn.pp.core.T;
import nn.pp.ext.devPref.DevicePrefs;
import nn.pp.logging.RemoteConsoleLogger;

public class AudioDialog implements ActionListener, AudioDialogAdapter {
   private JDialog dlg;
   private String parentTitle;
   private AudioCore audiocore;
   private AudioConnInProgressDialog progressDialog;
   private Logger logger;
   private AudioBean audioBean;
   private String targetPlaybackFormat;
   private String targetCaptureFormat;
   private DevicePrefs prefs;
   private JButton okButton;
   private JButton cancelButton;
   private JCheckBox playbackCheck;
   private JComboBox playbackDeviceCombo;
   private JComboBox playbackFormatCombo;
   private JLabel playbackFormatLabel;
   private JCheckBox captureCheck;
   private JComboBox captureDeviceCombo;
   private JComboBox captureFormatCombo;
   private JLabel captureFormatLabel;
   private boolean cancelled = false;
   private boolean autoClosed = false;
   private List<AudioDevice> playbackDevices;
   private List<AudioDevice> captureDevices;
   private boolean isFirstConnection = false;
   private static final String devicesMessage = "<html><font color=\"#0000ff\"><b><i>"
      + T._(
         "Note, to ensure proper operation attach audio devices to client PC<br>prior to launching browser. If you haven't already done so, please<br>close then reopen your browser after attaching the audio devices."
      )
      + "<i/></b></font></html>";
   private static final int matchLen = 30;
   private AudioAdapter audioListener = new AudioAdapter() {
      @Override
      public void audioConnected(boolean var1) {
         AudioDialog.this.savePreferences();
         AudioDialog.this.applyBufferSizes();
         AudioDialog.this.disposeProgressDialog();
         JOptionPane.showMessageDialog(
            AudioDialog.this.dlg,
            AudioDialog.this.isFirstConnection
               ? AudioMessages.audioSuccessfulConnectionMessage
               : AudioMessages.audioSuccessfulConnectionMessage + "\n" + AudioMessages.audioSharingHintMessage,
            T._("Successfully Connected"),
            1
         );
         AudioDialog.this.setVisible(false);
         AudioDialog.this.okButton.setEnabled(true);
         AudioDialog.this.cancelButton.setEnabled(true);
      }

      @Override
      public void receivedNotification(final INotificationEvent var1) {
         RemoteConsoleLogger.getInstance().getLogger().log(Level.INFO, "Received Notification:" + var1.toString());
         if (var1.getErrorCode() == -1862139899) {
            SwingUtilities.invokeLater(new Runnable() {
               @Override
               public void run() {
                  JOptionPane.showMessageDialog(AudioDialog.this.dlg, AudioMessages.audioUsbnotconnConnectionMessage, T._("Warning"), 2);
                  AudioDialog.this.okButton.setEnabled(true);
                  AudioDialog.this.cancelButton.setEnabled(true);
                  AudioDialog.this.setVisible(false);
               }
            });
         } else {
            SwingUtilities.invokeLater(new Runnable() {
               @Override
               public void run() {
                  AudioDialog.this.audioBean.notificationReceived(AudioDialog.this.dlg, var1, AudioDialog.this.dlg.getTitle());
                  if (var1.isQuit() || var1.isError()) {
                     AudioDialog.this.audiocore.disconnect();
                  }

                  AudioDialog.this.okButton.setEnabled(true);
                  AudioDialog.this.cancelButton.setEnabled(true);
               }
            });
         }

         AudioDialog.this.disposeProgressDialog();
      }
   };

   public AudioDialog(Frame var1, String var2, boolean var3, AudioCore var4, AudioBean var5, String var6, String var7, Logger var8) throws HeadlessException {
      this.dlg = new JDialog(var1, var2, var3);
      this.init(var1.getTitle(), var4, var5, var6, var7, var8);
   }

   public AudioDialog(JDialog var1, String var2) throws HeadlessException {
      this.dlg = var1;
      var1.setTitle(var2);
      this.init(null, null, null, null, null, null);
   }

   private void init(String var1, AudioCore var2, AudioBean var3, String var4, String var5, Logger var6) {
      this.parentTitle = var1;
      this.audiocore = var2;
      this.audioBean = var3;
      this.targetPlaybackFormat = var4;
      this.targetCaptureFormat = var5;
      this.logger = var6;
      if (var2 != null) {
         var2.addAudioEventListener(this.audioListener);
         var2.addNotificationListener(this.audioListener);
      }

      this.playbackDevices = AudioDevice.enumeratePlaybackDevices();
      this.captureDevices = AudioDevice.enumerateCaptureDevices();
      this.createGui();
      this.applyListeners();
   }

   public void setAudioBean(AudioBean var1) {
      this.audioBean = var1;
   }

   public void setPrefs(DevicePrefs var1) {
      this.prefs = var1;
   }

   public void setTargetFormats(String var1, String var2) {
      this.targetPlaybackFormat = var1;
      this.targetCaptureFormat = var2;
   }

   public void setAudioCore(AudioCore var1) {
      if (this.audiocore != null) {
         this.audiocore.removeAudioEventListener(this.audioListener);
         this.audiocore.removeNotificationListener(this.audioListener);
      }

      this.audiocore = var1;
      this.audiocore.addAudioEventListener(this.audioListener);
      this.audiocore.addNotificationListener(this.audioListener);
   }

   public void updateDevices() {
      this.playbackDevices = AudioDevice.enumeratePlaybackDevices();
      this.captureDevices = AudioDevice.enumerateCaptureDevices();
      this.updateDevicesUi();
   }

   public void dispose() {
      if (this.audiocore != null) {
         this.audiocore.removeAudioEventListener(this.audioListener);
         this.audiocore.removeNotificationListener(this.audioListener);
      }

      this.dlg.dispose();
   }

   protected void applyListeners() {
      this.okButton.addActionListener(this);
      this.cancelButton.addActionListener(this);
      this.playbackCheck.addActionListener(this);
      this.captureCheck.addActionListener(this);
      this.playbackDeviceCombo.addActionListener(this);
      this.captureDeviceCombo.addActionListener(this);
   }

   @Override
   public void actionPerformed(ActionEvent var1) {
      Object var2 = var1.getSource();
      if (var2 == this.cancelButton) {
         this.cancelled = true;
         this.setVisible(false);
      } else if (var2 == this.okButton) {
         this.performOk();
      } else if (var2 == this.playbackCheck || var2 == this.captureCheck) {
         this.checkEnabled();
      } else if (var2 == this.playbackDeviceCombo) {
         this.checkFormat(this.playbackDeviceCombo, this.playbackFormatCombo, this.targetPlaybackFormat);
      } else if (var2 == this.captureDeviceCombo) {
         this.checkFormat(this.captureDeviceCombo, this.captureFormatCombo, this.targetCaptureFormat);
      }
   }

   @Override
   public void setVisible(boolean var1) {
      if (var1 && this.playbackDevices.isEmpty() && this.captureDevices.isEmpty()) {
         this.audioBean.errorMessage(this.dlg.getOwner(), T._("No Audio devices found."), this.parentTitle != null ? this.parentTitle : "");
         this.autoClosed = true;
      } else {
         if (var1) {
            this.autoClosed = false;
            this.dlg.setLocationRelativeTo(this.dlg.getOwner());
         } else {
            this.audiocore.removeAudioEventListener(this.audioListener);
            this.audiocore.removeNotificationListener(this.audioListener);
         }

         this.dlg.setVisible(var1);
      }
   }

   public boolean isCancelled() {
      return this.cancelled;
   }

   @Override
   public boolean isAutoClosed() {
      return this.autoClosed;
   }

   @Override
   public boolean audioAvailable() {
      return !this.playbackDevices.isEmpty() || !this.captureDevices.isEmpty();
   }

   protected JPanel createMainPanel() {
      JLabel var1 = new JLabel(devicesMessage);
      JPanel var2 = new JPanel() {
         @Override
         public Dimension getPreferredSize() {
            Dimension var1x = super.getPreferredSize();
            return new Dimension(Math.max(300, (int)var1x.getWidth()), (int)var1x.getHeight());
         }
      };
      var2.setLayout(new SpringLayout());
      var2.setBorder(new TitledBorder(T._("Playback")));
      this.playbackCheck = new JCheckBox(T._("Connect Playback Device"));
      var2.add(this.playbackCheck);
      this.playbackDeviceCombo = new JComboBox();
      var2.add(this.playbackDeviceCombo);
      var2.add(this.playbackFormatLabel = new JLabel(T._("Format:")));
      this.playbackFormatCombo = new JComboBox();
      var2.add(this.playbackFormatCombo);
      SpringUtilities.makeCompactGrid(var2, 4, 1, 5, 0, 5, 5);
      JPanel var3 = new JPanel();
      var3.setLayout(new SpringLayout());
      var3.setBorder(new TitledBorder(T._("Recording")));
      this.captureCheck = new JCheckBox(T._("Connect Recording Device"));
      var3.add(this.captureCheck);
      this.captureDeviceCombo = new JComboBox();
      var3.add(this.captureDeviceCombo);
      var3.add(this.captureFormatLabel = new JLabel(T._("Format:")));
      this.captureFormatCombo = new JComboBox();
      var3.add(this.captureFormatCombo);
      SpringUtilities.makeCompactGrid(var3, 4, 1, 5, 0, 5, 5);
      JPanel var4 = new JPanel(new SpringLayout());
      var4.add(var1);
      var4.add(var2);
      var4.add(var3);
      SpringUtilities.makeCompactGrid(var4, 3, 1, 10, 10, 10, 10);
      this.updateDevicesUi();
      return var4;
   }

   private void updateDevicesUi() {
      DevicePrefs var1 = this.prefs != null ? this.prefs : (this.audioBean != null ? DevicePrefs.getNode(this.audioBean.getHost()) : null);
      this.playbackDeviceCombo.removeAllItems();
      this.captureDeviceCombo.removeAllItems();
      if (this.playbackDevices.isEmpty()) {
         this.playbackCheck.setEnabled(false);
         this.playbackCheck.setSelected(false);
      } else {
         this.playbackCheck.setEnabled(true);
         this.playbackCheck.setSelected(var1 != null ? var1.isConnectPlaybackDevice() : true);
         int var2 = 0;

         for (int var3 = 0; var3 < this.playbackDevices.size(); var3++) {
            AudioDevice var4 = this.playbackDevices.get(var3);
            this.playbackDeviceCombo.addItem(var4);
            if (var1 != null && this.deviceMatches(var4, var1.getPlaybackDevice())) {
               var2 = var3;
            }
         }

         this.playbackDeviceCombo.setSelectedIndex(var2);
      }

      this.captureCheck.setSelected(false);
      if (this.captureDevices.isEmpty()) {
         this.captureCheck.setEnabled(false);
      } else {
         this.captureCheck.setEnabled(this.targetCaptureFormat == null);
         this.captureCheck.setSelected(this.targetCaptureFormat == null && var1 != null ? var1.isConnectCaptureDevice() : false);
         int var5 = 0;

         for (int var6 = 0; var6 < this.captureDevices.size(); var6++) {
            AudioDevice var7 = this.captureDevices.get(var6);
            this.captureDeviceCombo.addItem(var7);
            if (var1 != null && this.deviceMatches(var7, var1.getCaptureDevice())) {
               var5 = var6;
            }
         }

         this.captureDeviceCombo.setSelectedIndex(var5);
      }

      this.checkEnabled();
      this.checkFormat(
         this.playbackDeviceCombo,
         this.playbackFormatCombo,
         this.targetPlaybackFormat != null ? this.targetPlaybackFormat : (var1 != null ? var1.getPlaybackFormat() : null)
      );
      this.checkFormat(
         this.captureDeviceCombo,
         this.captureFormatCombo,
         this.targetCaptureFormat != null ? this.targetCaptureFormat : (var1 != null ? var1.getCaptureFormat() : null)
      );
   }

   private boolean deviceMatches(AudioDevice var1, String var2) {
      if (var2 != null && !var2.isEmpty()) {
         String var3 = var1.toString();
         return var3.equals(var2) || var3.length() > 30 && var2.length() > 30 && var3.substring(0, 30).equals(var2.substring(0, 30));
      } else {
         return false;
      }
   }

   protected final JPanel createButtonPanel() {
      this.okButton = new JButton(T._("OK"));
      this.cancelButton = new JButton(T._("Cancel"));
      JPanel var1 = new JPanel(new FlowLayout(4));
      var1.add(this.okButton);
      var1.add(this.cancelButton);
      this.dlg.getRootPane().setDefaultButton(this.okButton);
      this.okButton.setPreferredSize(this.cancelButton.getPreferredSize());
      JPanel var2 = new JPanel(new SpringLayout());
      var2.add(var1);
      SpringUtilities.makeCompactGrid(var2, 1, 1, 0, 0, 10, 5);
      return var2;
   }

   private void createGui() {
      JPanel var1 = this.createButtonPanel();
      JPanel var2 = this.createMainPanel();
      this.dlg.setLayout(new BorderLayout());
      this.dlg.add(var2, "Center");
      this.dlg.add(var1, "Last");
      this.dlg.pack();
      this.dlg.setResizable(false);
   }

   private void checkEnabled() {
      this.okButton.setEnabled(this.captureCheck.isSelected() || this.playbackCheck.isSelected());
      this.playbackDeviceCombo.setEnabled(this.playbackCheck.isSelected());
      this.playbackFormatCombo.setEnabled(this.playbackCheck.isSelected() && this.targetPlaybackFormat == null);
      this.playbackFormatLabel.setEnabled(this.playbackCheck.isSelected() && this.targetPlaybackFormat == null);
      this.captureDeviceCombo.setEnabled(this.captureCheck.isSelected());
      this.captureFormatCombo.setEnabled(this.captureCheck.isSelected() && this.targetCaptureFormat == null);
      this.captureFormatLabel.setEnabled(this.captureCheck.isSelected() && this.targetCaptureFormat == null);
   }

   private void checkFormat(JComboBox var1, JComboBox var2, String var3) {
      Object var4 = var1.getSelectedItem();
      if (var4 != null) {
         AudioDevice var5 = (AudioDevice)var4;
         Object var6 = var2.getSelectedItem();
         var2.removeAllItems();
         int var7 = 0;
         List var8 = var5.getSupportedFormats();

         for (int var9 = 0; var9 < var8.size(); var9++) {
            AudioFormat var10 = (AudioFormat)var8.get(var9);
            var2.addItem(var10);
            if (var3 == null && var10 == var6 || var10.toShortString().equals(var3)) {
               var7 = var9;
            }
         }

         var2.setSelectedIndex(var7);
      }
   }

   private void disposeProgressDialog() {
      if (this.progressDialog != null && this.progressDialog.isShowing()) {
         this.progressDialog.setVisible(false);
         this.progressDialog.dispose();
      }

      this.dlg.toFront();
      this.dlg.requestFocus();
   }

   protected void performOk() {
      final AudioDevice var1 = null;
      final AudioDevice var2 = null;
      final AudioFormat var3 = null;
      final AudioFormat var4 = null;
      if (this.playbackCheck.isSelected()) {
         var1 = (AudioDevice)this.playbackDeviceCombo.getSelectedItem();
         var3 = (AudioFormat)this.playbackFormatCombo.getSelectedItem();
      }

      if (this.captureCheck.isSelected()) {
         var2 = (AudioDevice)this.captureDeviceCombo.getSelectedItem();
         var4 = (AudioFormat)this.captureFormatCombo.getSelectedItem();
      }

      if (var3 != null || var4 != null) {
         this.okButton.setEnabled(false);
         this.cancelButton.setEnabled(false);
         this.progressDialog = new AudioConnInProgressDialog(this.dlg, true);
         (new Thread() {
               @Override
               public void run() {
                  try {
                     AudioDialog.this.isFirstConnection = DeviceEnumerator.getConnectedDevices().isEmpty();
                     AudioDialog.this.connectAudio(var1, var3, var2, var4);
                  } catch (final Exception var2x) {
                     SwingUtilities.invokeLater(
                        new Runnable() {
                           @Override
                           public void run() {
                              if (AudioDialog.this.logger != null) {
                                 AudioDialog.this.logger.log(Level.SEVERE, T._("Unexpected Error happened during Audio connection."), (Throwable)var2x);
                              }

                              AudioDialog.this.audioBean
                                 .errorMessage(
                                    AudioDialog.this.dlg,
                                    var2x instanceof AudioException ? var2x.getMessage() : T._("Unexpected Error happened during Audio connection."),
                                    AudioDialog.this.dlg.getTitle()
                                 );
                              AudioDialog.this.disposeProgressDialog();
                              AudioDialog.this.okButton.setEnabled(true);
                              AudioDialog.this.cancelButton.setEnabled(true);
                           }
                        }
                     );
                  }
               }
            })
            .start();
      }
   }

   private void connectAudio(AudioDevice var1, AudioFormat var2, AudioDevice var3, AudioFormat var4) throws Exception {
      String var5 = this.audioBean.getPortId();
      int var6 = this.audioBean.getRfbSessionId();
      this.audioBean.setCaptureDevice(var3);
      this.audioBean.setCaptureFormat(var4);
      this.audioBean.setPlaybackDevice(var1);
      this.audioBean.setPlaybackFormat(var2);
      if (this.audioBean.getUsername() != null
         && this.audioBean.getPassword() != null
         && !this.audioBean.getUsername().isEmpty()
         && !this.audioBean.getPassword().isEmpty()) {
         this.audiocore
            .connectAudioWithUserLogin(
               this.audioBean.getHost(),
               this.audioBean.getPort(),
               this.audioBean.isSslMode(),
               var5,
               var6,
               this.audioBean.getMsindex(),
               var1,
               var2,
               var3,
               var4,
               this.audioBean.getUsername(),
               this.audioBean.getPassword()
            );
      } else if (this.audioBean.getEricKey() != null && !this.audioBean.getEricKey().isEmpty()) {
         this.audiocore
            .connectAudioWithEricKey(
               this.audioBean.getHost(),
               this.audioBean.getPort(),
               this.audioBean.isSslMode(),
               var5,
               var6,
               this.audioBean.getMsindex(),
               var1,
               var2,
               var3,
               var4,
               this.audioBean.getEricKey()
            );
      } else {
         if (this.audioBean.getRdmSession() == null || this.audioBean.getRdmSession().isEmpty()) {
            throw new AudioException(T._("Authentication parameters missing!"));
         }

         this.audiocore
            .connectAudioWithRdmSession(
               this.audioBean.getHost(),
               this.audioBean.getPort(),
               this.audioBean.isSslMode(),
               var5,
               var6,
               this.audioBean.getMsindex(),
               var1,
               var2,
               var3,
               var4,
               this.audioBean.getRdmSession(),
               this.audioBean.getProxyConnectionId(),
               this.audioBean.getProxyUseSSL()
            );
      }
   }

   private void savePreferences() {
      AudioDevice var1 = this.audioBean.getPlaybackDevice();
      AudioDevice var2 = this.audioBean.getCaptureDevice();
      AudioFormat var3 = this.audioBean.getPlaybackFormat();
      AudioFormat var4 = this.audioBean.getCaptureFormat();
      DevicePrefs var5 = this.prefs != null ? this.prefs : DevicePrefs.getNode(this.audioBean.getHost());
      if (var5 == null) {
         var5 = new DevicePrefs();
      }

      if (this.targetPlaybackFormat == null) {
         if (var1 != null && var3 != null) {
            var5.setConnectPlaybackDevice(true);
            var5.setPlaybackDevice(var1.toString());
            var5.setPlaybackFormat(var3.toShortString());
         } else {
            var5.setConnectPlaybackDevice(false);
         }
      }

      if (this.targetCaptureFormat == null) {
         if (var2 != null && var4 != null) {
            var5.setConnectCaptureDevice(true);
            var5.setCaptureDevice(var2.toString());
            var5.setCaptureFormat(var4.toShortString());
         } else {
            var5.setConnectCaptureDevice(false);
         }
      }

      var5.exportPreferences(this.audioBean.getHost());
   }

   private void applyBufferSizes() {
      DevicePrefs var1 = this.prefs != null ? this.prefs : DevicePrefs.getNode(this.audioBean.getHost());
      if (var1 != null) {
         try {
            int var2;
            if ((var2 = var1.getPlaybackBufferSize()) >= 0 && var2 != 120 && this.audioBean.getPlaybackFormat() != null) {
               this.audiocore.setPlaybackBufferSizeMs(var2);
            }

            if ((var2 = var1.getCaptureBufferSize()) >= 0 && var2 != 120 && this.audioBean.getCaptureFormat() != null) {
               this.audiocore.setCaptureBufferSizeMs(var2);
            }
         } catch (IOException var3) {
            this.logger.log(Level.WARNING, "Could not set buffer size.", (Throwable)var3);
         }
      }
   }
}
