package nn.pp.common.scan;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JComponent;
import javax.swing.Timer;
import nn.pp.common.RemoteConsoleParameters;
import nn.pp.core.INotificationEvent;
import nn.pp.core.NotificationListener;
import nn.pp.core.T;
import nn.pp.ext.devPref.DevicePrefs;
import nn.pp.logging.RemoteConsoleLogger;
import nn.pp.rccore.RCAdapter;
import nn.pp.rccore.RCCore;
import nn.pp.rccore.RCCoreFactory;
import nn.pp.rccore.RCException;

public class ScanConnector {
   private static final Dimension SCALE_DIMENSION = new Dimension(320, 240);
   private int scanInterval = 10000;
   private int switchInterval = 10000;
   private final Timer scanIntervalTimer;
   private final Timer switchIntervalTimer;
   private final List<RemoteConsoleParameters> remoteConsoleParamsList;
   private Iterator<RemoteConsoleParameters> iterator;
   private RemoteConsoleParameters currentRemoteConsoleParameter;
   private final Map<String, Integer> remoteConsoleParamsIndexes = new HashMap<>();
   private final List<ScanEventListener> listeners = new ArrayList<>();
   private RCCore rccore;
   private boolean paused = true;
   private static final Logger LOGGER = Logger.getLogger(ScanConnector.class.getName());
   private Random switchIntervalRandomizer = new Random();
   private int scanReferralId = -1;

   public ScanConnector(List<RemoteConsoleParameters> var1) {
      this.scanIntervalTimer = new Timer(this.scanInterval, new ScanConnector.ScanIntervalExpiredAction());
      this.switchIntervalTimer = new Timer(this.switchInterval, new ScanConnector.SwitchIntervalTimerExpired());
      this.scanIntervalTimer.setRepeats(false);
      this.switchIntervalTimer.setRepeats(false);
      this.remoteConsoleParamsList = Collections.unmodifiableList(var1);

      for (int var2 = 0; var2 < var1.size(); var2++) {
         this.remoteConsoleParamsIndexes.put(((RemoteConsoleParameters)var1.get(var2)).targetPortId, var2);
      }

      assert !var1.isEmpty();

      this.iterator = this.remoteConsoleParamsList.iterator();
   }

   public int getScanInterval() {
      return this.scanInterval;
   }

   public void setScanInterval(int var1) {
      this.scanInterval = var1;
      this.scanIntervalTimer.setInitialDelay(var1);
   }

   public int getSwitchInterval() {
      return this.switchInterval;
   }

   public void setSwitchInterval(int var1) {
      this.switchInterval = var1;
      this.switchIntervalRandomizer = new Random();
   }

   public void pause() {
      if (!this.paused) {
         this.paused = true;
         if (this.rccore != null) {
            if (this.scanIntervalTimer.isRunning()) {
               this.fireDisplayVideoEnd(this.getCurrentRemoteConsoleParameter().targetPortId, this.rccore.getSnapshot());
            } else {
               this.fireDisplayVideoEnd(this.getCurrentRemoteConsoleParameter().targetPortId, null);
            }

            this.cleanupRcCore();
         }

         this.scanIntervalTimer.stop();
         this.switchIntervalTimer.stop();
      }
   }

   public void resume() {
      if (this.paused) {
         this.paused = false;
         this.createAndConnectRcCore();
      }
   }

   public void resumeAfter(String var1) {
      if (this.paused) {
         Integer var2 = this.remoteConsoleParamsIndexes.get(var1);
         if (var2 != null) {
            var2 = var2 + 1;
            if (var2 > this.remoteConsoleParamsList.size()) {
               var2 = 0;
            }

            this.seekPort(var2);
            this.resume();
         }
      }
   }

   public void resumeAt(String var1) {
      if (this.paused) {
         Integer var2 = this.remoteConsoleParamsIndexes.get(var1);
         if (var2 != null) {
            this.seekPort(var2);
            this.resume();
         }
      }
   }

   public void stop() {
      this.pause();
   }

   public void start(int var1) {
      this.scanReferralId = var1;
      this.resume();
   }

   private RemoteConsoleParameters getNextRemoteConsoleParameter() {
      if (!this.iterator.hasNext()) {
         this.iterator = this.remoteConsoleParamsList.iterator();
      }

      this.currentRemoteConsoleParameter = this.iterator.next();
      return this.getCurrentRemoteConsoleParameter();
   }

   private RemoteConsoleParameters getCurrentRemoteConsoleParameter() {
      return this.currentRemoteConsoleParameter;
   }

   private void seekPort(int var1) {
      this.iterator = this.remoteConsoleParamsList.listIterator(var1);
   }

   private void createAndConnectRcCore() {
      assert this.rccore == null;

      RemoteConsoleParameters var1 = this.getNextRemoteConsoleParameter();
      this.rccore = RCCoreFactory.loadGraphicalReadOnlyRCCore(RemoteConsoleLogger.getInstance().getLogger());
      this.rccore.setScaleToFit(true, true, SCALE_DIMENSION);
      this.setConnectionParams(var1);
      this.rccore.addNotificationListener(new ScanConnector.VideoStateHandler(this.rccore));
      this.rccore.addNotificationListener(new ScanConnector.NotificationHandler(this.rccore));
      ScanConnector.ConnectionEventHandler var2 = new ScanConnector.ConnectionEventHandler(this.rccore);
      this.rccore.addConnectionEventListener(var2, 4);
      this.rccore.addVideoEventListener(var2, 32);
      this.fireVideoComponentAvailable(var1.targetPortId, this.rccore.getRCJComponent());

      try {
         if (var1.username != null && var1.password != null) {
            this.rccore.connectRCWithUserLogin(var1.getURLCompatibleRemoteHost(), var1.tcpPort, var1.ssl, var1.targetPortId, var1.username, var1.password);
         } else if (var1.ericKey != null) {
            this.rccore.connectRCWithEricKey(var1.getURLCompatibleRemoteHost(), var1.tcpPort, var1.ssl, var1.targetPortId, var1.ericKey);
         } else {
            if (var1.rdmSession == null) {
               throw new RCException(T._("Authentication parameters missing!"));
            }

            this.rccore
               .connectRCWithRdmSession(
                  var1.getURLCompatibleRemoteHost(), var1.tcpPort, var1.ssl, var1.targetPortId, var1.rdmSession, var1.proxyConnectionId, var1.proxyUseSSL
               );
         }
      } catch (IOException var4) {
         LOGGER.log(Level.SEVERE, "Exception on connection");
         this.abortScan();
      } catch (RCException var5) {
         LOGGER.log(Level.SEVERE, "Exception on connection");
         this.abortScan();
      }
   }

   private void setConnectionParams(RemoteConsoleParameters var1) {
      DevicePrefs var2 = DevicePrefs.getNode(var1.host);
      if (var2 != null) {
         boolean var3 = false;
         if (var2.getG2ConnectionSpeed() == null && var2.getG2ColorDepth() != null && var2.getG2Smoothing() != null) {
            var3 = true;
         }

         this.rccore.setConnectionProperties(var3, var2.getG2ConnectionSpeed(), var2.getG2ColorDepth(), var2.getG2Smoothing());
      }

      HashMap var4 = new HashMap();
      var4.put(RCCore.ClientSessionInitProperties.SCAN_REFEENCE_ID, this.scanReferralId + "");
      this.rccore.setClientSessionInitProperties(var4);
   }

   private void abortScan() {
      this.cleanup();
      this.fireScanAbortedOnConnectError();
   }

   private void abortScan(INotificationEvent var1) {
      this.cleanup();
      this.fireScanAborted(var1);
   }

   private void cleanup() {
      this.scanIntervalTimer.stop();
      this.switchIntervalTimer.stop();
      this.cleanupRcCore();
   }

   private void cleanupRcCore() {
      if (this.rccore != null) {
         this.rccore.disconnect();
         this.rccore.dispose();
         this.rccore = null;
      }
   }

   public boolean addScanEventListener(ScanEventListener var1) {
      return this.addScanEventListener_0(var1);
   }

   private boolean addScanEventListener_0(ScanEventListener var1) {
      return !this.listeners.contains(var1) ? this.listeners.add(var1) : false;
   }

   public boolean removeScanEventListener(ScanEventListener var1) {
      return this.listeners.remove(var1);
   }

   private void fireVideoComponentAvailable(String var1, JComponent var2) {
      for (ScanEventListener var4 : this.listeners) {
         var4.videoComponentAvailable(var1, var2);
      }
   }

   private void fireDisplayVideoStart(String var1, boolean var2) {
      for (ScanEventListener var4 : this.listeners) {
         var4.displayVideoStart(var1, var2);
      }
   }

   private void fireDisplayVideoEnd(String var1, Image var2) {
      for (ScanEventListener var4 : this.listeners) {
         var4.displayVideoEnd(var1, var2);
      }
   }

   private void fireDisplayVideoError(String var1, int var2) {
      for (ScanEventListener var4 : this.listeners) {
         var4.displayVideoError(var1, var2);
      }
   }

   private void fireDisplayVideoCommunicationError(String var1) {
      for (ScanEventListener var3 : this.listeners) {
         var3.displayVideoCommunicationError(var1);
      }
   }

   private void fireScanAborted(INotificationEvent var1) {
      for (ScanEventListener var3 : this.listeners) {
         var3.scanAborted(var1);
      }
   }

   private void fireScanAbortedOnConnectError() {
      for (ScanEventListener var2 : this.listeners) {
         var2.scanAbortedOnConnectError();
      }
   }

   private void fireOsdMessageReceived(String var1, String var2) {
      for (ScanEventListener var4 : this.listeners) {
         var4.osdMessageReceived(var1, var2);
      }
   }

   private void restartSwitchIntervalTimer(int var1) {
      this.switchIntervalTimer.setInitialDelay(var1);
      this.switchIntervalTimer.restart();
   }

   private class ConnectionEventHandler extends RCAdapter {
      private final RCCore owningRccore;

      public ConnectionEventHandler(RCCore var2) {
         this.owningRccore = var2;
      }

      @Override
      public void disconnected(Exception var1) {
         if (ScanConnector.this.rccore == this.owningRccore) {
            ScanConnector.LOGGER.log(Level.INFO, "communication error, continuing scan", (Throwable)var1);
            ScanConnector.this.fireDisplayVideoCommunicationError(ScanConnector.this.getCurrentRemoteConsoleParameter().targetPortId);
            ScanConnector.this.cleanupRcCore();
            ScanConnector.this.scanIntervalTimer.stop();

            assert !ScanConnector.this.switchIntervalTimer.isRunning();

            ScanConnector.this.restartSwitchIntervalTimer(ScanConnector.this.switchIntervalRandomizer.nextInt(ScanConnector.this.switchInterval + 1));
         }
      }

      @Override
      public void osdMessageReceived(String var1, boolean var2) {
         if (ScanConnector.this.rccore == this.owningRccore) {
            ScanConnector.this.fireOsdMessageReceived(ScanConnector.this.getCurrentRemoteConsoleParameter().targetPortId, var1);
         }
      }
   }

   private class NotificationHandler implements NotificationListener {
      private final RCCore owningRccore;

      public NotificationHandler(RCCore var2) {
         this.owningRccore = var2;
      }

      @Override
      public void receivedNotification(INotificationEvent var1) {
         if (ScanConnector.this.rccore == this.owningRccore) {
            switch (var1.getErrorCode()) {
               case 268435457:
               case 268566529:
               case 268566530:
               case 268566531:
               case 268566532:
               case 268566533:
               case 268566534:
               case 268697601:
               case 268697604:
                  ScanConnector.LOGGER.log(Level.INFO, "Nofication received, continuing scan : " + var1.getErrorCode());
                  ScanConnector.this.fireDisplayVideoError(ScanConnector.this.getCurrentRemoteConsoleParameter().targetPortId, var1.getErrorCode());
                  ScanConnector.this.cleanupRcCore();

                  assert !ScanConnector.this.scanIntervalTimer.isRunning();

                  assert !ScanConnector.this.switchIntervalTimer.isRunning();

                  ScanConnector.this.restartSwitchIntervalTimer(ScanConnector.this.switchIntervalRandomizer.nextInt(ScanConnector.this.switchInterval + 1));
                  break;
               case 268435459:
               case 268697602:
               case 268697603:
               case 268828673:
               case 302383106:
                  ScanConnector.LOGGER.log(Level.INFO, "Nofication received, aborting scan : " + var1.getErrorCode());
                  ScanConnector.this.abortScan(var1);
            }
         }
      }

      @Override
      public void textNotification(String var1) {
      }
   }

   private class ScanIntervalExpiredAction implements ActionListener {
      private ScanIntervalExpiredAction() {
      }

      @Override
      public void actionPerformed(ActionEvent var1) {
         ScanConnector.this.scanIntervalTimer.stop();
         ScanConnector.this.fireDisplayVideoEnd(ScanConnector.this.getCurrentRemoteConsoleParameter().targetPortId, ScanConnector.this.rccore.getSnapshot());
         ScanConnector.this.cleanupRcCore();

         assert !ScanConnector.this.switchIntervalTimer.isRunning();

         ScanConnector.this.restartSwitchIntervalTimer(ScanConnector.this.switchInterval);
      }
   }

   private class SwitchIntervalTimerExpired implements ActionListener {
      private SwitchIntervalTimerExpired() {
      }

      @Override
      public void actionPerformed(ActionEvent var1) {
         ScanConnector.this.switchIntervalTimer.stop();
         ScanConnector.this.createAndConnectRcCore();
      }
   }

   private class VideoStateHandler implements NotificationListener {
      private final RCCore owningRccore;

      public VideoStateHandler(RCCore var2) {
         this.owningRccore = var2;
      }

      @Override
      public void receivedNotification(INotificationEvent var1) {
         if (ScanConnector.this.rccore == this.owningRccore && var1.getErrorCode() == 302120971) {
            ScanConnector.this.fireDisplayVideoStart(ScanConnector.this.getCurrentRemoteConsoleParameter().targetPortId, false);

            assert !ScanConnector.this.scanIntervalTimer.isRunning();

            ScanConnector.this.scanIntervalTimer.restart();
            ScanConnector.this.rccore.removeNotificationListener(this);
         }
      }

      @Override
      public void textNotification(String var1) {
      }
   }
}
