package nn.pp.core.impl;

import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.MessageFormat;
import java.util.logging.Level;
import java.util.logging.Logger;
import nn.pp.core.T;

public abstract class ProtocolHandler<E extends Exception> extends Thread {
   protected DeviceConnector connector;
   protected MonitoringDataInputStream is;
   protected MonitoringDataOutputStream os;
   protected Logger logger;
   protected String host;
   protected String username;
   protected String password;
   protected String httpSessionID;
   protected String rdmSessionID;
   protected String portID;
   protected boolean shouldRun = false;
   protected boolean connected = false;
   protected boolean inInitialHandshake = false;
   private MessageDigest digest;
   protected static final int socketTimeoutInHandshake = 60000;
   protected int versionMajor;
   protected int versionMinor;

   protected static String getVersionString(int var0, int var1, char var2) {
      String var3 = "" + var0;
      if (var0 < 10) {
         var3 = "0" + var3;
      }

      String var4 = "" + var1;
      if (var1 < 10) {
         var4 = "0" + var4;
      }

      return new String(var3 + var2 + var4);
   }

   protected abstract String getProtocolName();

   protected abstract void loadPdus();

   protected abstract void processInitialHandshake(int var1) throws IOException, E;

   protected abstract void negotiateProtocolVersion() throws IOException, E;

   protected abstract void processProtocol() throws IOException, E;

   protected abstract void disconnected(Exception var1);

   protected abstract E loadException(String var1);

   public void init(DeviceConnector var1, Logger var2) throws IOException {
      this.connector = var1;
      this.is = var1.getInputStream();
      this.os = var1.getOutputStream();
      this.host = var1.getHost();
      this.logger = var2;
      this.loadPdus();
   }

   public void connect(String var1, String var2, String var3, String var4, String var5) throws IOException, E {
      this.portID = var1;
      this.username = var2;
      this.password = var3;
      this.httpSessionID = var4;
      this.rdmSessionID = var5;
      this.enterInitialHandshake();
      if (this.versionMajor == -1 || this.versionMinor == -1) {
         this.negotiateProtocolVersion();
      }

      this.shouldRun = true;
      this.start();
      this.connected = true;
   }

   public void close() {
      this.shouldRun = false;
   }

   public void dispose() {
      this.shouldRun = false;

      try {
         this.join(1000L);
         if (this.isAlive()) {
            this.logger.log(Level.WARNING, T._("Protocol thread still alive, killing forcefully!"));
            this.stop();
         }

         if (this.connector != null) {
            this.connector = null;
         }

         if (this.is != null) {
            this.is = null;
         }

         if (this.os != null) {
            this.os = null;
         }

         if (this.digest != null) {
            this.digest = null;
         }
      } catch (InterruptedException var2) {
      }
   }

   protected void enterInitialHandshake() throws IOException {
      this.inInitialHandshake = true;
      this.connector.setSocketTimeout(60000);
      this.connector.setTcpNoDelay(true);
   }

   protected void initialHandshakeFinished() throws IOException {
      this.inInitialHandshake = false;
      this.connector.setSocketTimeout(0);
   }

   @Override
   public void run() {
      this.logger.log(Level.INFO, MessageFormat.format(T._("{0} handler thread started."), this.getProtocolName()));

      try {
         this.processProtocol();
      } catch (Exception var2) {
         if (this.shouldRun) {
            this.logger.log(Level.SEVERE, MessageFormat.format(T._("Error occured in {0} protocol handling"), this.getProtocolName()), (Throwable)var2);
            this.disconnected(var2);
         }
      }

      this.logger.log(Level.INFO, MessageFormat.format(T._("{0} handler thread finished."), this.getProtocolName()));
   }

   protected int readServerMessageType() throws IOException {
      return this.is.readUnsignedByte();
   }

   public String binToHex(byte[] var1) {
      String var2 = "";

      for (int var3 = 0; var3 < var1.length; var3++) {
         int var4 = var1[var3] & 255;
         if (var4 <= 15) {
            var2 = var2 + "0";
         }

         var2 = var2 + Integer.toHexString(var4).toUpperCase();
      }

      return var2;
   }

   public String getChallengeResponse(byte[] var1, String var2) throws IOException, E {
      try {
         this.digest = MessageDigest.getInstance(var2);
      } catch (NoSuchAlgorithmException var5) {
         throw this.loadException(T._("Could not find " + var2 + " algorithm."));
      }

      this.digest.update(var1);
      this.digest.update(this.httpSessionID.getBytes("ISO-8859-1"), 0, this.httpSessionID.length());
      byte[] var3 = this.digest.digest();
      return this.binToHex(var3);
   }
}
