package nn.pp.rccore.scan.impl.rsp;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.EventListener;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.event.EventListenerList;
import nn.pp.core.impl.ProtocolHandler;
import nn.pp.rccore.scan.ScanCoreException;
import nn.pp.rccore.scan.ScanSessionEventsListener;
import nn.pp.rccore.scan.impl.commands.AuthHttpIDGetChallenge;
import nn.pp.rccore.scan.impl.commands.AuthNamePassRequest;
import nn.pp.rccore.scan.impl.commands.AuthRDMRequest;

public class RSPHandler extends ProtocolHandler<ScanCoreException> implements ClientCommandsExecutor, EventsListenerManager {
   private volatile RSPProtoHandler<?> protoHandler;
   private final EventListenerList listenersList = new EventListenerList();
   private static final Logger LOGGER = Logger.getLogger(RSPHandler.class.getName());

   public RSPHandler() {
      this.versionMajor = -1;
      this.versionMinor = -1;
   }

   @Override
   protected void disconnected(Exception var1) {
      this.protoHandler.handleDisconnection();

      for (ScanSessionEventsListener var3 : this.getListeners(ScanSessionEventsListener.class)) {
         var3.disconnected(var1);
      }
   }

   @Override
   protected String getProtocolName() {
      return "RSP";
   }

   protected ScanCoreException loadException(String var1) {
      return null;
   }

   @Override
   protected void loadPdus() {
   }

   @Override
   protected void negotiateProtocolVersion() throws IOException, ScanCoreException {
      new RSPHelloMessage().writeMessage(this.os);
      RSPVersionMessage var1 = new RSPVersionMessage().readMessage(this.is);
      this.loadProtocolHandler(var1);
      new RSPVersionAcceptMessage(var1.getVersionMajor(), var1.getVersionMinor()).writeMessage(this.os);
      this.initialHandshakeFinished();
      this.protoHandler.setClientCommandsExecutor(this);
      this.protoHandler.setEventsListenerManager(this);
   }

   private void loadProtocolHandler(RSPVersionMessage var1) throws ScanCoreException {
      String var2 = RSPHandler.class.getPackage().getName() + "." + var1.getVersionAsString() + "." + "RSPProtoHandler" + var1.getVersionAsString();
      LOGGER.log(Level.INFO, "Loading RSP Protocol Handler impl class with name - " + var2);

      Class var3;
      try {
         var3 = Class.forName(var2);
      } catch (ClassNotFoundException var14) {
         throw new ScanCoreException(var14);
      }

      Class var4;
      try {
         var4 = var3.asSubclass(RSPProtoHandler.class);
      } catch (ClassCastException var13) {
         throw new ScanCoreException(var13);
      }

      Constructor var5 = null;

      try {
         var5 = var4.getConstructor();
      } catch (SecurityException var11) {
         throw new ScanCoreException(var11);
      } catch (NoSuchMethodException var12) {
         throw new ScanCoreException(var12);
      }

      try {
         this.protoHandler = (RSPProtoHandler<?>)var5.newInstance();
      } catch (IllegalArgumentException var7) {
         throw new ScanCoreException(var7);
      } catch (InstantiationException var8) {
         throw new ScanCoreException(var8);
      } catch (IllegalAccessException var9) {
         throw new ScanCoreException(var9);
      } catch (InvocationTargetException var10) {
         throw new ScanCoreException(var10);
      }
   }

   @Override
   protected void processInitialHandshake(int var1) throws IOException, ScanCoreException {
   }

   @Override
   protected void processProtocol() throws IOException, ScanCoreException {
      this.sendAuthenticationMessage();

      while (this.shouldRun) {
         this.handleMessage(this.protoHandler);
      }
   }

   private void sendAuthenticationMessage() throws IOException, ScanCoreException {
      if (this.username != null && this.password != null) {
         this.executeClientCommands(new AuthNamePassRequest(this.username, this.password));
      } else if (this.rdmSessionID != null) {
         this.executeClientCommands(new AuthRDMRequest(this.rdmSessionID));
      } else {
         if (this.httpSessionID == null) {
            throw new ScanCoreException("No Authentication information available");
         }

         this.executeClientCommands(new AuthHttpIDGetChallenge(this.httpSessionID));
      }
   }

   private <V> void handleMessage(RSPProtoHandler<V> var1) throws IOException, ScanCoreException {
      this.executeCommand(var1, var1.getMessageToCommandProducer(this.readServerMessageType()));
   }

   private <V> void executeCommand(RSPProtoHandler<V> var1, MessageToCommandProducer<V> var2) throws IOException, ScanCoreException {
      if (var2 == null) {
         throw new ScanCoreException("Protocol Error: Unknown Protocol message received");
      } else {
         var2 = var2.readMessage(this.is);
         LOGGER.log(Level.INFO, "Received Command " + var2);
         var1.executeCommand(var2);
      }
   }

   @Override
   public <T> void executeClientCommands(SupportsVisitor<T> var1) {
      LOGGER.log(Level.INFO, "Sending command for execution - " + var1);

      try {
         this.protoHandler.executeClientCommands(this.os, this.is, var1);
      } catch (IOException var3) {
         this.close();
         this.disconnected(var3);
      } catch (ScanCoreException var4) {
         this.close();
         this.disconnected(var4);
      } catch (RuntimeException var5) {
         this.close();
         this.disconnected(var5);
         throw var5;
      }
   }

   @Override
   public <T extends EventListener> void addListener(Class<T> var1, T var2) {
      this.listenersList.add(var1, var2);
   }

   @Override
   public <T extends EventListener> List<T> getListeners(Class<T> var1) {
      return Arrays.asList((T[])this.listenersList.getListeners(var1));
   }

   @Override
   public <T extends EventListener> void removeListener(Class<T> var1, T var2) {
      this.listenersList.remove(var1, var2);
   }
}
