package nn.pp.vmcore.impl;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.logging.Level;
import java.util.logging.Logger;
import nn.pp.core.T;
import nn.pp.vmcore.VMCore;
import nn.pp.vmcore.VMException;

public class RedirectedImage extends RedirectedObject {
   protected File imageFile;
   protected String imageFileName;
   protected RandomAccessFile file;
   protected static final int ISO_SECTOR_SIZE = 2048;
   protected static final int FLOPPY_SECTOR_SIZE = 512;
   protected static final int REMOVABLE_SECTOR_SIZE = 512;
   protected static final int FIXED_SECTOR_SIZE = 512;
   protected int sectorSize;
   protected long sectorCount;
   protected long lastSectorNo;
   private static final int TAGLEN = 5;
   private static final int ISO9660_MAGIC_POS = 32769;
   private static final String ISOTAG = "CD001";

   public RedirectedImage(File var1, boolean var2, Logger var3) {
      super(var2, var3);
      this.imageFile = var1;
      this.imageFileName = var1.getAbsolutePath();
   }

   @Override
   public synchronized void open() throws IOException {
      this.file = new RandomAccessFile(this.imageFile, this.readOnly ? "r" : "rw");
      if (this.file == null) {
         throw new IOException(T._("Could not open file"));
      }
   }

   @Override
   public synchronized void close() {
      if (this.file != null) {
         try {
            this.file.close();
         } catch (IOException var2) {
         }
      }

      this.file = null;
   }

   @Override
   public synchronized void determineGeometry() throws IOException {
      if (this.isIso(this.file)) {
         this.driveType = VMCore.DriveType.CDROM;
         this.sectorSize = 2048;
      } else if (this.isFloppy(this.file)) {
         this.driveType = VMCore.DriveType.FLOPPY;
         this.sectorSize = 512;
      } else if (this.isRemovable(this.file)) {
         this.driveType = VMCore.DriveType.REMOVABLE;
         this.sectorSize = 512;
      } else {
         if (!this.isFixed(this.file)) {
            throw new IOException(T._("Invalid image file found."));
         }

         this.driveType = VMCore.DriveType.HARD_DISK_FULL;
         this.sectorSize = 512;
      }

      this.sectorCount = this.file.length() / this.sectorSize;
      this.lastSectorNo = this.sectorCount - 1L;
   }

   @Override
   public synchronized long getLastSectorNo() {
      return this.lastSectorNo;
   }

   @Override
   public synchronized int getSectorSize() {
      return this.sectorSize;
   }

   @Override
   protected synchronized void lockAccess(boolean var1) throws VMException {
   }

   @Override
   public synchronized void readSectors(long var1, long var3, byte[] var5) throws IOException {
      this.readPartitionSectors(var1, var3, var5, 0);
   }

   @Override
   public synchronized void writeSectors(long var1, long var3, byte[] var5) throws IOException, VMException {
      if (this.readOnly) {
         throw new VMException(T._("Writing not supported"));
      } else {
         this.writePartitionSectors(var1, var3, var5, 0);
      }
   }

   protected synchronized boolean isFloppy(RandomAccessFile var1) throws IOException {
      long var2 = var1.length();
      if (var2 <= 2949120L && var2 % 512L == 0L) {
         this.logger.log(Level.FINE, "Image file is floppy image");
         return true;
      } else {
         this.logger.log(Level.FINE, "Image file is not a floppy image!");
         return false;
      }
   }

   protected synchronized boolean isIso(RandomAccessFile var1) throws IOException {
      long var2 = var1.length();
      if (var2 < 32774L || var2 % 512L != 0L) {
         this.logger.log(Level.FINE, "Image file is not a ISO image, size doesn't match!");
      }

      byte[] var4 = new byte[5];

      try {
         long var5 = var1.getFilePointer();
         var1.seek(32769L);
         var1.readFully(var4, 0, 5);
         var1.seek(var5);
      } catch (IOException var7) {
         return false;
      }

      String var8 = new String(var4);
      if (!"CD001".equals(var8)) {
         this.logger.log(Level.FINE, "Image file is not a ISO image, tag doesn't match (" + var8 + " - " + "CD001" + ")");
         return false;
      } else {
         this.logger.log(Level.FINE, "Image file is ISO image");
         return true;
      }
   }

   protected synchronized boolean isRemovable(RandomAccessFile var1) throws IOException {
      long var2 = var1.length();
      if (var2 % 512L != 0L) {
         this.logger.log(Level.FINE, "Image file is not a removable image!");
         return false;
      } else {
         this.logger.log(Level.FINE, "Image file is removable image");
         return true;
      }
   }

   protected synchronized boolean isFixed(RandomAccessFile var1) throws IOException {
      return false;
   }

   protected void readPartitionSectors(long var1, long var3, byte[] var5, int var6) throws IOException {
      this.file.seek(var1 * this.sectorSize);
      this.file.readFully(var5, var6, (int)(var3 * this.sectorSize));
   }

   protected void writePartitionSectors(long var1, long var3, byte[] var5, int var6) throws IOException, VMException {
      this.file.seek(var1 * this.sectorSize);
      this.file.write(var5, var6, (int)(var3 * this.sectorSize));
   }
}
