package nn.pp.vmcore.impl;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.CRC32;
import nn.pp.core.T;
import nn.pp.core.impl.MonitoringDataOutputStream;
import nn.pp.vmcore.VMException;

public class RedirectedImageWithPartitionTable extends RedirectedImage {
   private static final int FAT16_PARTITION_TYPE = 6;
   private static final int FAT32_PARTITION_TYPE = 11;
   private static final int NTFS_PARTITION_TYPE = 7;
   private static final UUID BASIC_DATA_PARTITION_GUID = UUID.fromString("EBD0A0A2-B9E5-4433-87C0-68B6B72699C7");
   private static final int FAT_MAGIC_POS = 54;
   private static final int NTFS_MAGIC_POS = 3;
   private static final String FAT16_MAGIC_STRING = "FAT16   ";
   private static final String FAT32_MAGIC_STRING = "FAT32   ";
   private static final String NTFS_MAGIC_STRING = "NTFS    ";
   private static final long MAX_32_BIT_SECTORNO = 4294967295L;
   private static final int HEADER_SECTORS = 256;
   private static final int FOOTER_SECTORS = 64;
   private boolean useMbr = false;
   private byte[] header = new byte[131072];
   private byte[] footer = new byte[32768];

   public RedirectedImageWithPartitionTable(File var1, boolean var2, Logger var3) {
      super(var1, var2, var3);
   }

   @Override
   public synchronized void determineGeometry() throws IOException {
      super.determineGeometry();
      this.createMbrIfNeeded();
   }

   @Override
   public synchronized void readSectors(long var1, long var3, byte[] var5) throws IOException {
      this.doReadSectors(var1, var3, var5, 0);
   }

   @Override
   public synchronized void writeSectors(long var1, long var3, byte[] var5) throws IOException, VMException {
      if (this.readOnly) {
         throw new VMException(T._("Writing not supported"));
      } else {
         this.doWriteSectors(var1, var3, var5, 0);
      }
   }

   private long crc32(byte[] var1, int var2, int var3) {
      CRC32 var4 = new CRC32();
      var4.update(var1, var2, var3);
      return var4.getValue();
   }

   private long crc32(byte[] var1) {
      return this.crc32(var1, 0, var1.length);
   }

   private boolean matchMagic(String var1, int var2, String var3) throws IOException {
      byte[] var4 = new byte[var3.length()];

      try {
         long var5 = this.file.getFilePointer();
         this.file.seek(var2);
         this.file.readFully(var4, 0, var3.length());
         this.file.seek(var5);
      } catch (IOException var7) {
         return false;
      }

      String var8 = new String(var4);
      if (!var3.equals(var8)) {
         this.logger.log(Level.FINE, "Image file is not a " + var1 + " image, tag doesn't match (" + var8 + " - " + var3 + ")");
         return false;
      } else {
         this.logger.log(Level.FINE, "Image file is " + var1 + " image");
         return true;
      }
   }

   private boolean isFat16() throws IOException {
      return this.matchMagic("FAT16", 54, "FAT16   ");
   }

   private boolean isFat32() throws IOException {
      return this.matchMagic("FAT32", 54, "FAT32   ");
   }

   private boolean isNtfs() throws IOException {
      return this.matchMagic("NTFS", 3, "NTFS    ");
   }

   private void createMbrIfNeeded() throws IOException {
      if (this.isRemovable(this.file)) {
         byte var1;
         if (this.isFat16()) {
            var1 = 6;
         } else if (this.isFat32()) {
            var1 = 11;
         } else {
            if (!this.isNtfs()) {
               return;
            }

            var1 = 7;
         }

         if (this.sectorCount > 4294967295L) {
            this.createGpt();
         } else {
            this.createMbr(var1);
         }

         this.lastSectorNo += 320L;
         this.useMbr = true;
      }
   }

   private void createMbr(byte var1) throws IOException {
      RedirectedImageWithPartitionTable.Mbr var2 = new RedirectedImageWithPartitionTable.Mbr();
      var2.partTable[0].status = -128;
      var2.partTable[0].chsStart[0] = -1;
      var2.partTable[0].chsStart[1] = -1;
      var2.partTable[0].chsStart[2] = -1;
      var2.partTable[0].partType = var1;
      var2.partTable[0].chsLast[0] = -1;
      var2.partTable[0].chsLast[1] = -1;
      var2.partTable[0].chsLast[2] = -1;
      var2.partTable[0].lbaFirstBlock = 256L;
      var2.partTable[0].size = this.sectorCount;
      byte[] var3 = var2.toByteArray();
      System.arraycopy(var3, 0, this.header, 0, var3.length);
   }

   private void createGpt() throws IOException {
      RedirectedImageWithPartitionTable.PrimaryGpt var1 = new RedirectedImageWithPartitionTable.PrimaryGpt();
      long var2 = this.sectorCount + 256L + 64L;
      long var4 = var2 > 4294967295L ? 4294967295L : var2;
      var1.protectiveMbr.partTable[0].status = 0;
      var1.protectiveMbr.partTable[0].chsStart[0] = 0;
      var1.protectiveMbr.partTable[0].chsStart[1] = 32;
      var1.protectiveMbr.partTable[0].chsStart[2] = 0;
      var1.protectiveMbr.partTable[0].partType = -18;
      var1.protectiveMbr.partTable[0].chsLast[0] = -1;
      var1.protectiveMbr.partTable[0].chsLast[1] = -1;
      var1.protectiveMbr.partTable[0].chsLast[2] = -1;
      var1.protectiveMbr.partTable[0].lbaFirstBlock = 1L;
      var1.protectiveMbr.partTable[0].size = var4;
      var1.header.myLba = 1L;
      var1.header.alternateLba = var2 - 1L;
      var1.header.firstUsableLba = 256L;
      var1.header.lastUsableLba = var2 - 64L - 1L;
      var1.header.diskGuid = UUID.randomUUID();
      var1.header.partitionEntryLba = 2L;
      var1.partitionTable.partTable[0].partitionTypeGuid = BASIC_DATA_PARTITION_GUID;
      var1.partitionTable.partTable[0].uniquePartitionGuid = UUID.randomUUID();
      var1.partitionTable.partTable[0].startingLba = 256L;
      var1.partitionTable.partTable[0].endingLba = this.sectorCount + 256L - 1L;
      var1.partitionTable.partTable[0].attributes = 0L;
      var1.partitionTable.partTable[0].partitionName = "Raritan Virtual Media";
      RedirectedImageWithPartitionTable.SecondaryGpt var6 = new RedirectedImageWithPartitionTable.SecondaryGpt(var1);
      var6.header.myLba = var1.header.alternateLba;
      var6.header.alternateLba = 1L;
      var6.header.partitionEntryLba = var2 - 33L;
      long var7 = this.crc32(var1.partitionTable.toByteArray());
      var1.header.partitionEntryArrayCrc32 = var7;
      var6.header.partitionEntryArrayCrc32 = var7;
      long var9 = this.crc32(var1.header.toByteArray(), 0, var1.header.headerSize);
      var1.header.headerCrc32 = var9;
      long var11 = this.crc32(var6.header.toByteArray(), 0, var6.header.headerSize);
      var6.header.headerCrc32 = var11;
      byte[] var13 = var1.toByteArray();
      System.arraycopy(var13, 0, this.header, 0, var13.length);
      var13 = var6.toByteArray();
      System.arraycopy(var13, 0, this.footer, this.footer.length - var13.length, var13.length);
   }

   private void doReadSectors(long var1, long var3, byte[] var5, int var6) throws IOException {
      if (!this.useMbr) {
         this.readPartitionSectors(var1, var3, var5, var6);
      } else if (var1 < 256L) {
         int var15 = (int)Math.min(var3, 256L - var1);
         System.arraycopy(this.header, (int)(var1 * this.sectorSize), var5, var6, var15 * this.sectorSize);
         var3 -= var15;
         var1 += var15;
         var6 += var15 * this.sectorSize;
         if (var3 != 0L) {
            this.doReadSectors(var1, var3, var5, var6);
         }
      } else if (var1 < this.sectorCount + 256L) {
         int var14 = (int)Math.min(var3, this.sectorCount + 256L - var1);
         this.readPartitionSectors(var1 - 256L, var14, var5, var6);
         var3 -= var14;
         var1 += var14;
         var6 += var14 * this.sectorSize;
         if (var3 != 0L) {
            this.doReadSectors(var1, var3, var5, var6);
         }
      } else if (var1 <= this.lastSectorNo) {
         int var7 = (int)Math.min(var3, this.lastSectorNo + 1L - var1);
         System.arraycopy(this.footer, (int)(var1 - this.sectorCount - 256L) * this.sectorSize, var5, var6, var7 * this.sectorSize);
      }
   }

   private void doWriteSectors(long var1, long var3, byte[] var5, int var6) throws IOException, VMException {
      if (!this.useMbr) {
         this.writePartitionSectors(var1, var3, var5, var6);
      } else if (var1 < 256L) {
         int var11 = (int)Math.min(var3, 256L - var1);
         var3 -= var11;
         var1 += var11;
         var6 += var11 * this.sectorSize;
         if (var3 != 0L) {
            this.doWriteSectors(var1, var3, var5, var6);
         }
      } else if (var1 <= this.lastSectorNo) {
         if (var1 < this.sectorCount + 256L) {
            int var7 = (int)Math.min(var3, this.sectorCount + 256L - var1);
            this.writePartitionSectors(var1 - 256L, var7, var5, var6);
         }
      }
   }

   private abstract class Gpt extends RedirectedImageWithPartitionTable.Writable {
      private Gpt() {
      }

      byte[] uuidToByteArray(UUID var1) {
         byte[] var2 = new byte[16];
         long var3 = var1.getMostSignificantBits();
         long var5 = var1.getLeastSignificantBits();

         for (int var7 = 0; var7 < 8; var7++) {
            var2[var7] = (byte)(var3 >>> 8 * (7 - var7));
         }

         for (int var8 = 8; var8 < 16; var8++) {
            var2[var8] = (byte)(var5 >>> 8 * (7 - var8));
         }

         byte var9 = var2[0];
         var2[0] = var2[3];
         var2[3] = var9;
         var9 = var2[1];
         var2[1] = var2[2];
         var2[2] = var9;
         var9 = var2[4];
         var2[4] = var2[5];
         var2[5] = var9;
         var9 = var2[6];
         var2[6] = var2[7];
         var2[7] = var9;
         return var2;
      }

      class GptHeader extends RedirectedImageWithPartitionTable.Writable {
         byte[] signature = "EFI PART".getBytes();
         int revision = 65536;
         int headerSize = 92;
         long headerCrc32;
         long myLba;
         long alternateLba;
         long firstUsableLba;
         long lastUsableLba;
         UUID diskGuid;
         long partitionEntryLba;
         int numberOfPartitionEntries = 128;
         int sizeOfPartitionEntries = 128;
         long partitionEntryArrayCrc32;
         byte[] reservedBlock = new byte[420];

         private GptHeader() {
         }

         private GptHeader(RedirectedImageWithPartitionTable.Gpt.GptHeader var2) {
            this.headerCrc32 = var2.headerCrc32;
            this.myLba = var2.myLba;
            this.alternateLba = var2.alternateLba;
            this.firstUsableLba = var2.firstUsableLba;
            this.lastUsableLba = var2.lastUsableLba;
            this.diskGuid = var2.diskGuid;
            this.partitionEntryLba = var2.partitionEntryLba;
            this.partitionEntryArrayCrc32 = var2.partitionEntryArrayCrc32;
         }

         @Override
         void copyToStream(MonitoringDataOutputStream var1) throws IOException {
            var1.write(this.signature);
            var1.writeUnsignedIntLE(this.revision);
            var1.writeUnsignedIntLE(this.headerSize);
            var1.writeUnsignedIntLE(this.headerCrc32);
            var1.writeUnsignedIntLE(0L);
            var1.writeUnsignedLongLE(this.myLba);
            var1.writeUnsignedLongLE(this.alternateLba);
            var1.writeUnsignedLongLE(this.firstUsableLba);
            var1.writeUnsignedLongLE(this.lastUsableLba);
            var1.write(Gpt.this.uuidToByteArray(this.diskGuid));
            var1.writeUnsignedLongLE(this.partitionEntryLba);
            var1.writeUnsignedIntLE(this.numberOfPartitionEntries);
            var1.writeUnsignedIntLE(this.sizeOfPartitionEntries);
            var1.writeUnsignedIntLE(this.partitionEntryArrayCrc32);
            var1.write(this.reservedBlock);
         }
      }

      class GptPartTable extends RedirectedImageWithPartitionTable.Writable {
         private RedirectedImageWithPartitionTable.Gpt.GptPartTable.Entry[] partTable = new RedirectedImageWithPartitionTable.Gpt.GptPartTable.Entry[128];

         private GptPartTable() {
            for (int var2 = 0; var2 < this.partTable.length; var2++) {
               this.partTable[var2] = new RedirectedImageWithPartitionTable.Gpt.GptPartTable.Entry();
            }
         }

         private GptPartTable(RedirectedImageWithPartitionTable.Gpt.GptPartTable var2) {
            for (int var3 = 0; var3 < this.partTable.length; var3++) {
               this.partTable[var3] = new RedirectedImageWithPartitionTable.Gpt.GptPartTable.Entry(var2.partTable[var3]);
            }
         }

         @Override
         void copyToStream(MonitoringDataOutputStream var1) throws IOException {
            for (int var2 = 0; var2 < this.partTable.length; var2++) {
               this.partTable[var2].copyToStream(var1);
            }
         }

         class Entry extends RedirectedImageWithPartitionTable.Writable {
            UUID partitionTypeGuid;
            UUID uniquePartitionGuid;
            long startingLba;
            long endingLba;
            long attributes;
            String partitionName;
            byte[] dummyUuid = new byte[16];

            private Entry() {
            }

            private Entry(RedirectedImageWithPartitionTable.Gpt.GptPartTable.Entry var2) {
               this.partitionTypeGuid = var2.partitionTypeGuid;
               this.uniquePartitionGuid = var2.uniquePartitionGuid;
               this.startingLba = var2.startingLba;
               this.endingLba = var2.endingLba;
               this.attributes = var2.attributes;
               this.partitionName = var2.partitionName;
            }

            @Override
            void copyToStream(MonitoringDataOutputStream var1) throws IOException {
               if (this.partitionTypeGuid != null) {
                  var1.write(Gpt.this.uuidToByteArray(this.partitionTypeGuid));
               } else {
                  var1.write(this.dummyUuid);
               }

               if (this.uniquePartitionGuid != null) {
                  var1.write(Gpt.this.uuidToByteArray(this.uniquePartitionGuid));
               } else {
                  var1.write(this.dummyUuid);
               }

               var1.writeUnsignedLongLE(this.startingLba);
               var1.writeUnsignedLongLE(this.endingLba);
               var1.writeUnsignedLongLE(this.attributes);
               byte[] var2 = this.partitionName != null ? this.partitionName.getBytes() : new byte[0];

               for (int var3 = 0; var3 < 36; var3++) {
                  var1.write(var3 < var2.length ? var2[var3] : 0);
                  var1.write(0);
               }
            }
         }
      }
   }

   private class Mbr extends RedirectedImageWithPartitionTable.Writable {
      byte[] codeArea = new byte[440];
      int diskSignature;
      RedirectedImageWithPartitionTable.Mbr.Entry[] partTable = new RedirectedImageWithPartitionTable.Mbr.Entry[4];

      Mbr() {
         for (int var2 = 0; var2 < this.partTable.length; var2++) {
            this.partTable[var2] = new RedirectedImageWithPartitionTable.Mbr.Entry();
         }
      }

      @Override
      void copyToStream(MonitoringDataOutputStream var1) throws IOException {
         var1.write(this.codeArea);
         var1.writeUnsignedIntLE(this.diskSignature);
         var1.writeUnsignedShortLE(0);

         for (int var2 = 0; var2 < this.partTable.length; var2++) {
            this.partTable[var2].copyToStream(var1);
         }

         var1.writeUnsignedShortLE(43605);
      }

      class Entry extends RedirectedImageWithPartitionTable.Writable {
         static final byte BOOTABLE = -128;
         static final byte NOTBOOTABLE = 0;
         byte status;
         byte[] chsStart = new byte[3];
         byte partType;
         byte[] chsLast = new byte[3];
         long lbaFirstBlock;
         long size;

         @Override
         void copyToStream(MonitoringDataOutputStream var1) throws IOException {
            var1.write(this.status);
            var1.write(this.chsStart);
            var1.write(this.partType);
            var1.write(this.chsLast);
            var1.writeUnsignedIntLE(this.lbaFirstBlock);
            var1.writeUnsignedIntLE(this.size);
         }
      }
   }

   private class PrimaryGpt extends RedirectedImageWithPartitionTable.Gpt {
      RedirectedImageWithPartitionTable.Mbr protectiveMbr = RedirectedImageWithPartitionTable.this.new Mbr();
      RedirectedImageWithPartitionTable.Gpt.GptHeader header = new RedirectedImageWithPartitionTable.Gpt.GptHeader();
      RedirectedImageWithPartitionTable.Gpt.GptPartTable partitionTable = new RedirectedImageWithPartitionTable.Gpt.GptPartTable();

      private PrimaryGpt() {
      }

      @Override
      void copyToStream(MonitoringDataOutputStream var1) throws IOException {
         this.protectiveMbr.copyToStream(var1);
         this.header.copyToStream(var1);
         this.partitionTable.copyToStream(var1);
      }
   }

   private class SecondaryGpt extends RedirectedImageWithPartitionTable.Gpt {
      RedirectedImageWithPartitionTable.Gpt.GptPartTable partitionTable;
      RedirectedImageWithPartitionTable.Gpt.GptHeader header;

      SecondaryGpt(RedirectedImageWithPartitionTable.PrimaryGpt var2) {
         this.partitionTable = new RedirectedImageWithPartitionTable.Gpt.GptPartTable(var2.partitionTable);
         this.header = new RedirectedImageWithPartitionTable.Gpt.GptHeader(var2.header);
      }

      @Override
      void copyToStream(MonitoringDataOutputStream var1) throws IOException {
         this.partitionTable.copyToStream(var1);
         this.header.copyToStream(var1);
      }
   }

   private abstract class Writable {
      private Writable() {
      }

      abstract void copyToStream(MonitoringDataOutputStream var1) throws IOException;

      byte[] toByteArray() throws IOException {
         ByteArrayOutputStream var1 = new ByteArrayOutputStream(512);
         MonitoringDataOutputStream var2 = new MonitoringDataOutputStream(var1);
         this.copyToStream(var2);
         var2.close();
         return var1.toByteArray();
      }
   }
}
