package com.sun.deploy.security;

import com.sun.deploy.resources.ResourceManager;
import com.sun.deploy.util.Trace;
import java.io.IOException;
import java.security.AccessController;
import java.security.KeyStore;
import java.security.Principal;
import java.security.PrivilegedActionException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import sun.security.util.DerInputStream;
import sun.security.util.DerValue;
import sun.security.x509.NetscapeCertTypeExtension;

public class CertUtils {
   private static final String OID_BASIC_CONSTRAINTS = "2.5.29.19";
   private static final String OID_KEY_USAGE = "2.5.29.15";
   private static final String OID_EXTENDED_KEY_USAGE = "2.5.29.37";
   private static final String OID_NETSCAPE_CERT_TYPE = "2.16.840.1.113730.1.1";
   private static final String OID_EKU_ANY_USAGE = "2.5.29.37.0";
   private static final String OID_EKU_CODE_SIGNING = "1.3.6.1.5.5.7.3.3";
   private static final String OID_EKU_SERVER_AUTH = "1.3.6.1.5.5.7.3.1";
   private static final String OID_EKU_CLIENT_AUTH = "1.3.6.1.5.5.7.3.2";
   private static final String OID_EKU_TIME_STAMPING = "1.3.6.1.5.5.7.3.8";
   private static final String NSCT_OBJECT_SIGNING_CA = "object_signing_ca";
   private static final String NSCT_OBJECT_SIGNING = "object_signing";
   private static final String NSCT_SSL_CA = "ssl_ca";
   private static final String NSCT_S_MIME_CA = "s_mime_ca";
   private static final String NSCT_S_MIME = "s_mime";
   private static final String NSCT_SSL_CLIENT = "ssl_client";
   private static final String NSCT_SSL_SERVER = "ssl_server";

   public static KeyStore createEmptyKeyStore() {
      KeyStore var0 = null;

      try {
         var0 = KeyStore.getInstance("JKS");
         var0.load(null, null);
      } catch (Exception var2) {
         Trace.ignoredException(var2);
      }

      return var0;
   }

   public static void checkUsageForCodeSigning(X509Certificate var0, int var1) throws CertificateException, IOException {
      checkUsageForCodeSigning(var0, var1, false);
   }

   public static void checkUsageForCodeSigning(X509Certificate var0, int var1, boolean var2) throws CertificateException, IOException {
      Object var3 = null;
      Set var4 = var0.getCriticalExtensionOIDs();
      if (var4 == null) {
         var4 = Collections.EMPTY_SET;
      }

      if (!checkBasicConstraintsForCodeSigning(var0, var4, var1)) {
         Trace.msgSecurityPrintln("trustdecider.check.basicconstraints");
         var3 = ResourceManager.getMessage("trustdecider.check.basicconstraints");
         throw new CertificateException((String)var3);
      } else {
         if (var1 == 0) {
            if (!checkLeafKeyUsageForCodeSigning(var0, var4, var2)) {
               Trace.msgSecurityPrintln("trustdecider.check.leafkeyusage");
               var3 = ResourceManager.getMessage("trustdecider.check.leafkeyusage");
               throw new CertificateException((String)var3);
            }
         } else if (!checkSignerKeyUsage(var0, var4)) {
            Trace.msgSecurityPrintln("trustdecider.check.signerkeyusage");
            var3 = ResourceManager.getMessage("trustdecider.check.signerkeyusage");
            throw new CertificateException((String)var3);
         }

         if (!var4.isEmpty()) {
            Trace.msgSecurityPrintln("trustdecider.check.extensions");
            var3 = ResourceManager.getMessage("trustdecider.check.extensions");
            throw new CertificateException((String)var3);
         }
      }
   }

   private static boolean checkBasicConstraintsForCodeSigning(X509Certificate var0, Set var1, int var2) throws CertificateException, IOException {
      var1.remove("2.5.29.19");
      var1.remove("2.16.840.1.113730.1.1");
      if (var2 == 0) {
         return true;
      } else {
         if (var0.getExtensionValue("2.5.29.19") == null) {
            if (var0.getExtensionValue("2.16.840.1.113730.1.1") == null) {
               Trace.msgSecurityPrintln("trustdecider.check.basicconstraints.extensionvalue");
               return false;
            }

            if (!getNetscapeCertTypeBit(var0, "object_signing_ca")) {
               Trace.msgSecurityPrintln("trustdecider.check.basicconstraints.certtypebit");
               return false;
            }
         } else {
            if (var0.getExtensionValue("2.16.840.1.113730.1.1") != null
               && (getNetscapeCertTypeBit(var0, "ssl_ca") || getNetscapeCertTypeBit(var0, "s_mime_ca") || getNetscapeCertTypeBit(var0, "object_signing_ca"))
               && !getNetscapeCertTypeBit(var0, "object_signing_ca")) {
               Trace.msgSecurityPrintln("trustdecider.check.basicconstraints.bitvalue");
               return false;
            }

            int var3 = var0.getBasicConstraints();
            if (var3 < 0) {
               Trace.msgSecurityPrintln("trustdecider.check.basicconstraints.enduser");
               return false;
            }

            if (var2 - 1 > var3) {
               Trace.msgSecurityPrintln("trustdecider.check.basicconstraints.pathlength");
               return false;
            }
         }

         return true;
      }
   }

   private static boolean checkLeafKeyUsageForCodeSigning(X509Certificate var0, Set var1, boolean var2) throws CertificateException, IOException {
      var1.remove("2.5.29.15");
      boolean[] var3 = var0.getKeyUsage();
      if (var3 != null) {
         if (var3.length == 0) {
            Trace.msgSecurityPrintln("trustdecider.check.leafkeyusage.length");
            return false;
         }

         boolean var4 = var3[0];
         if (!var4) {
            Trace.msgSecurityPrintln("trustdecider.check.leafkeyusage.digitalsignature");
            return false;
         }
      }

      List var6 = X509Util.getExtendedKeyUsage(var0);
      Set var5 = var0.getNonCriticalExtensionOIDs();
      if (var6 != null && (var1.contains("2.5.29.37") || var5.contains("2.5.29.37"))) {
         var1.remove("2.5.29.37");
         if (var2) {
            if (!var6.contains("2.5.29.37.0") && !var6.contains("1.3.6.1.5.5.7.3.8")) {
               Trace.msgSecurityPrintln("trustdecider.check.leafkeyusage.tsaextkeyusageinfo");
               return false;
            }
         } else if (!var6.contains("2.5.29.37.0") && !var6.contains("1.3.6.1.5.5.7.3.3")) {
            Trace.msgSecurityPrintln("trustdecider.check.leafkeyusage.extkeyusageinfo");
            return false;
         }
      }

      if (var0.getExtensionValue("2.16.840.1.113730.1.1") != null && !getNetscapeCertTypeBit(var0, "object_signing")) {
         Trace.msgSecurityPrintln("trustdecider.check.leafkeyusage.certtypebit");
         return false;
      } else {
         return true;
      }
   }

   private static boolean checkSignerKeyUsage(X509Certificate var0, Set var1) throws CertificateException, IOException {
      var1.remove("2.5.29.15");
      boolean[] var2 = var0.getKeyUsage();
      if (var2 == null || var2.length >= 6 && var2[5]) {
         List var3 = X509Util.getExtendedKeyUsage(var0);
         Set var4 = var0.getNonCriticalExtensionOIDs();
         if (var3 != null && (var1.contains("2.5.29.37") || var4.contains("2.5.29.37"))) {
            var1.remove("2.5.29.37");
            if (!var3.contains("2.5.29.37.0") && !var3.contains("1.3.6.1.5.5.7.3.3")) {
               Trace.msgSecurityPrintln("trustdecider.check.signerkeyusage.keyusage");
               return false;
            }
         }

         return true;
      } else {
         Trace.msgSecurityPrintln("trustdecider.check.signerkeyusage.lengthandbit");
         return false;
      }
   }

   private static boolean getNetscapeCertTypeBit(X509Certificate var0, String var1) throws CertificateException, IOException {
      byte[] var2 = var0.getExtensionValue("2.16.840.1.113730.1.1");
      if (var2 == null) {
         return false;
      } else {
         DerInputStream var3 = new DerInputStream(var2);
         byte[] var4 = var3.getOctetString();
         var4 = new DerValue(var4).getUnalignedBitString().toByteArray();
         NetscapeCertTypeExtension var5 = new NetscapeCertTypeExtension(var4);
         Boolean var6 = (Boolean)var5.get(var1);
         return var6;
      }
   }

   public static boolean isIssuerOf(X509Certificate var0, X509Certificate var1) {
      Principal var2 = var0.getIssuerDN();
      Principal var3 = var1.getSubjectDN();
      return var2.equals(var3);
   }

   private static String extractFromQuote(String var0, String var1) {
      if (var0 == null) {
         return null;
      } else {
         int var2 = var0.indexOf(var1);
         int var3 = 0;
         if (var2 >= 0) {
            var2 += var1.length();
            if (var0.charAt(var2) == '"') {
               var3 = var0.indexOf(34, ++var2);
            } else {
               var3 = var0.indexOf(44, var2);
            }

            return var3 < 0 ? var0.substring(var2) : var0.substring(var2, var3);
         } else {
            return null;
         }
      }
   }

   public static String extractSubjectAliasName(X509Certificate var0) {
      String var1 = ResourceManager.getMessage("config.unknownSubject");

      try {
         Principal var2 = var0.getSubjectDN();
         String var3 = var2.getName();
         var1 = extractFromQuote(var3, "CN=");
         if (var1 == null) {
            String var4 = extractFromQuote(var3, "O=");
            String var5 = extractFromQuote(var3, "OU=");
            if (var4 != null || var5 != null) {
               MessageFormat var6 = new MessageFormat(ResourceManager.getMessage("config.certShowOOU"));
               Object[] var7 = new Object[]{var4, var5};
               if (var4 == null) {
                  var7[0] = "";
               }

               if (var5 == null) {
                  var7[1] = "";
               }

               var1 = var6.format(var7);
            }
         }

         if (var1 == null) {
            var1 = ResourceManager.getMessage("config.unknownSubject");
         }
      } catch (Exception var8) {
      }

      return var1;
   }

   public static String extractIssuerAliasName(X509Certificate var0) {
      String var1 = ResourceManager.getMessage("config.unknownIssuer");

      try {
         Principal var2 = var0.getIssuerDN();
         String var3 = var2.getName();
         var1 = extractFromQuote(var3, "CN=");
         if (var1 == null) {
            String var4 = extractFromQuote(var3, "O=");
            String var5 = extractFromQuote(var3, "OU=");
            if (var4 != null || var5 != null) {
               MessageFormat var6 = new MessageFormat(ResourceManager.getMessage("config.certShowOOU"));
               Object[] var7 = new Object[]{var4, var5};
               if (var4 == null) {
                  var7[0] = "";
               }

               if (var5 == null) {
                  var7[1] = "";
               }

               var1 = var6.format(var7);
            }
         }

         if (var1 == null) {
            var1 = ResourceManager.getMessage("config.unknownIssuer");
         }
      } catch (Exception var8) {
      }

      return var1;
   }

   static long getFileLastModified(String var0) {
      long var1 = 0L;

      try {
         var1 = AccessController.doPrivileged(new CertUtils$1(var0));
      } catch (PrivilegedActionException var4) {
         Trace.securityPrintException(var4);
      }

      return var1;
   }
}
