package sun.plugin.util;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.security.AccessController;
import java.util.Properties;
import sun.security.action.GetPropertyAction;

public abstract class DeploymentConfig {
   private Properties configFileProps = null;

   public DeploymentConfig() {
      this.configFileProps = this.loadConfigProperties();
      new File(this.getUserHome()).mkdirs();
      new File(this.getSystemHome()).mkdirs();
      new File(this.getCacheDirectory()).mkdirs();
      new File(this.getTempDirectory()).mkdirs();
      new File(this.getLogDirectory()).mkdirs();
      new File(this.getSecurityDirectory()).mkdirs();
      new File(this.getExtensionDirectory()).mkdirs();
      new File(new File(this.getPropertiesFile()).getParent()).mkdirs();
   }

   public String getCacheDirectory() {
      String var1 = this.configFileProps.getProperty("deployment.user.cachedir");
      if (var1 != null && var1.startsWith("file:")) {
         try {
            return new File(URI.create(var1)).toString();
         } catch (IllegalArgumentException var3) {
         }
      }

      return this.getUserHome() + File.separator + "cache";
   }

   public String getExtensionDirectory() {
      String var1 = this.configFileProps.getProperty("deployment.user.extdir");
      if (var1 != null && var1.startsWith("file:")) {
         try {
            return new File(URI.create(var1)).toString();
         } catch (IllegalArgumentException var3) {
         }
      }

      return this.getUserHome() + File.separator + "ext";
   }

   protected abstract String getInfrastructurePath();

   public String getJavaHome() {
      return AccessController.doPrivileged(new GetPropertyAction("java.home"));
   }

   public String getJsseCACertStore() {
      String var1 = this.configFileProps.getProperty("deployment.system.jssecacerts");
      if (var1 != null && var1.startsWith("file:")) {
         try {
            return new File(URI.create(var1)).toString();
         } catch (IllegalArgumentException var3) {
         }
      }

      String var2 = this.getSystemHome() + File.separator + "security" + File.separator + "jssecacerts";
      if (!new File(var2).exists()) {
         var2 = this.getSystemHome() + File.separator + "security" + File.separator + "cacerts";
         if (!new File(var2).exists()) {
            var2 = this.getJavaHome() + File.separator + "lib" + File.separator + "security" + File.separator + "jssecacerts";
            if (!new File(var2).exists()) {
               var2 = this.getJavaHome() + File.separator + "lib" + File.separator + "security" + File.separator + "cacerts";
            }
         }
      }

      return var2;
   }

   public String getLogDirectory() {
      String var1 = this.configFileProps.getProperty("deployment.user.logdir");
      if (var1 != null && var1.startsWith("file:")) {
         try {
            return new File(URI.create(var1)).toString();
         } catch (IllegalArgumentException var3) {
         }
      }

      return this.getUserHome() + File.separator + "log";
   }

   public String getPropertiesFile() {
      return this.getUserHome() + File.separator + this.getPropertiesFilename();
   }

   protected String getPropertiesFilename() {
      return "deployment.properties";
   }

   public String getRootCACertStore() {
      String var1 = this.configFileProps.getProperty("deployment.system.cacerts");
      if (var1 != null && var1.startsWith("file:")) {
         try {
            return new File(URI.create(var1)).toString();
         } catch (IllegalArgumentException var3) {
         }
      }

      String var2 = this.getSystemHome() + File.separator + "security" + File.separator + "cacerts";
      if (!new File(var2).exists()) {
         var2 = this.getJavaHome() + File.separator + "lib" + File.separator + "security" + File.separator + "cacerts";
      }

      return var2;
   }

   public String getSecurityDirectory() {
      return this.getUserHome() + File.separator + "security";
   }

   public String getSystemHome() {
      return this.getSystemProfilePath() + File.separator + this.getInfrastructurePath();
   }

   protected abstract String getSystemProfilePath();

   public String getTempDirectory() {
      return this.getCacheDirectory() + File.separator + "tmp";
   }

   public String getTrustedJsseCertStore() {
      String var1 = this.configFileProps.getProperty("deployment.user.jssecerts");
      if (var1 != null && var1.startsWith("file:")) {
         try {
            return new File(URI.create(var1)).toString();
         } catch (IllegalArgumentException var3) {
         }
      }

      return this.getUserHome() + File.separator + "security" + File.separator + "deployment.jssecerts";
   }

   public String getTrustedSigningCertStore() {
      String var1 = this.configFileProps.getProperty("deployment.user.certs");
      if (var1 != null && var1.startsWith("file:")) {
         try {
            return new File(URI.create(var1)).toString();
         } catch (IllegalArgumentException var3) {
         }
      }

      return this.getUserHome() + File.separator + "security" + File.separator + "deployment.certs";
   }

   public String getUserHome() {
      return this.getUserProfilePath() + File.separator + this.getInfrastructurePath();
   }

   protected abstract String getUserProfilePath();

   public URL getSystemSecurityPolicyURL() {
      String var1 = this.configFileProps.getProperty("deployment.system.security.policy");
      if (var1 != null) {
         try {
            return URI.create(var1).toURL();
         } catch (IllegalArgumentException var4) {
         } catch (MalformedURLException var5) {
         }
      }

      try {
         return new File(this.getSystemHome() + File.separator + "security" + File.separator + "java.policy").toURI().toURL();
      } catch (MalformedURLException var3) {
         return null;
      }
   }

   public URL getUserSecurityPolicyURL() {
      String var1 = this.configFileProps.getProperty("deployment.user.security.policy");
      if (var1 != null) {
         try {
            return URI.create(var1).toURL();
         } catch (IllegalArgumentException var4) {
         } catch (MalformedURLException var5) {
         }
      }

      try {
         return new File(this.getUserHome() + File.separator + "security" + File.separator + "java.policy").toURI().toURL();
      } catch (MalformedURLException var3) {
         return null;
      }
   }

   public Properties getConfigProperties() {
      Properties var1 = new Properties();
      var1.setProperty("deployment.user.profile", this.getUserProfilePath());
      var1.setProperty("deployment.system.profile", this.getSystemProfilePath());
      var1.setProperty("deployment.user.home", this.getUserHome());
      var1.setProperty("deployment.system.home", this.getSystemHome());
      var1.setProperty("deployment.user.cachedir", this.getCacheDirectory());
      var1.setProperty("deployment.user.logdir", this.getLogDirectory());
      var1.setProperty("deployment.user.extdir", this.getExtensionDirectory());
      var1.setProperty("deployment.user.tmpdir", this.getTempDirectory());
      var1.setProperty("deployment.user.security.policy", this.getUserSecurityPolicyURL().toString());
      var1.setProperty("deployment.system.security.policy", this.getSystemSecurityPolicyURL().toString());
      var1.setProperty("deployment.user.certs", this.getTrustedSigningCertStore());
      var1.setProperty("deployment.user.jssecerts", this.getTrustedJsseCertStore());
      var1.setProperty("deployment.system.cacerts", this.getRootCACertStore());
      var1.setProperty("deployment.system.jssecacerts", this.getJsseCACertStore());
      return var1;
   }

   private Properties loadConfigProperties() {
      Properties var1 = new Properties();

      try {
         FileInputStream var2 = new FileInputStream(this.getUserHome() + File.separator + "deployment.config");
         BufferedInputStream var3 = new BufferedInputStream(var2);
         var1.load(var3);
         var3.close();
         var2.close();
      } catch (FileNotFoundException var9) {
      } catch (IOException var10) {
      }

      try {
         Properties var11 = new Properties();
         FileInputStream var12 = new FileInputStream(this.getPropertiesFile());
         BufferedInputStream var4 = new BufferedInputStream(var12);
         var11.load(var4);
         var4.close();
         var12.close();
         String var5 = var11.getProperty("deployment.user.cachedir");
         if (var5 != null) {
            var1.setProperty("deployment.user.cachedir", var5);
         }

         String var6 = var11.getProperty("deployment.user.logdir");
         if (var6 != null) {
            var1.setProperty("deployment.user.logdir", var6);
         }
      } catch (FileNotFoundException var7) {
      } catch (IOException var8) {
      }

      return var1;
   }
}
